"""Patient models."""

from django.db import models
from django.utils.translation import gettext_lazy as _

from dfhir.base import choices as base_choices
from dfhir.base.models import (
    Address,
    Attachment,
    BaseReference,
    CodeableConcept,
    Communication,
    ContactPoint,
    HumanName,
    Identifier,
    OrganizationReference,
    Period,
    TimeStampedModel,
)

from . import choices


class PatientRelatedPersonReference(BaseReference):
    """Patient related person reference model."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.CASCADE,
        related_name="patient_related_person_reference_identifier",
        null=True,
    )
    patient = models.ForeignKey(
        "Patient",
        on_delete=models.CASCADE,
        related_name="patient_related_person_reference_patient",
        null=True,
    )
    related_person = models.ForeignKey(
        "relatedpersons.RelatedPerson",
        on_delete=models.CASCADE,
        related_name="patient_related_person_reference_related_person",
        null=True,
    )


class PatientGroupReference(BaseReference):
    """Patient group reference model."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.CASCADE,
        related_name="patient_group_reference_identifier",
        null=True,
    )
    patient = models.ForeignKey(
        "Patient",
        on_delete=models.CASCADE,
        related_name="patient_group_reference_patient",
        null=True,
    )
    group = models.ForeignKey(
        "groups.Group",
        on_delete=models.CASCADE,
        related_name="patient_group_reference_group",
        null=True,
    )


class PatientContact(TimeStampedModel):
    """Patient contact model."""

    name = models.ForeignKey(
        HumanName,
        on_delete=models.CASCADE,
        related_name="patient_contact_name",
        null=True,
    )
    gender = models.CharField(
        max_length=20, choices=base_choices.GenderChoices.choices, null=True, blank=True
    )
    relationship = models.ManyToManyField(
        CodeableConcept, related_name="patient_contact_relationship", blank=True
    )
    address = models.ForeignKey(
        Address,
        on_delete=models.CASCADE,
        related_name="patient_contact_address",
        null=True,
    )
    period = models.ForeignKey(
        Period,
        on_delete=models.CASCADE,
        related_name="patient_contact_period",
        null=True,
    )
    organization = models.ForeignKey(
        OrganizationReference,
        on_delete=models.DO_NOTHING,
        null=True,
        blank=True,
        related_name="patient_contact_organization",
    )
    telecom = models.ManyToManyField(
        ContactPoint, related_name="patient_contact_telecom", blank=True
    )


class PatientLink(TimeStampedModel):
    """Patient link model."""

    other = models.ForeignKey(
        "PatientRelatedPersonReference",
        on_delete=models.CASCADE,
        related_name="patient_link_other",
        null=True,
    )
    type = models.CharField(
        max_length=20, choices=choices.PatientLinkChoices.choices, null=True, blank=True
    )


class Patient(TimeStampedModel):
    """Patient model."""

    identifier = models.ManyToManyField(
        Identifier, related_name="patient_identifier", blank=True
    )
    name = models.ManyToManyField(HumanName, related_name="patient_name", blank=True)
    telecom = models.ManyToManyField(
        ContactPoint, related_name="patient_telecom", blank=True
    )
    birth_date = models.DateField(null=True, blank=True)
    gender = models.CharField(
        max_length=20, choices=base_choices.GenderChoices.choices, null=True, blank=True
    )
    middle_name = models.CharField(max_length=255, blank=True, null=True)
    email = models.EmailField(_("email address"), unique=True, null=True)
    deceased_boolean = models.BooleanField(default=True)
    deceased_date_time = models.DateTimeField(null=True, blank=True)
    active = models.BooleanField(default=True)
    marital_status = models.ForeignKey(
        CodeableConcept,
        on_delete=models.DO_NOTHING,
        null=True,
        blank=True,
        related_name="patient_marital_status",
    )
    communication = models.ManyToManyField(
        Communication, related_name="patient_communication", blank=True
    )
    multiple_birth_boolean = models.BooleanField(default=False)
    multiple_birth_integer = models.IntegerField(null=True, blank=True)
    photo = models.ManyToManyField(Attachment, related_name="patient_photo", blank=True)
    contact = models.ManyToManyField(
        PatientContact, related_name="patient_contact", blank=True
    )
    general_practitioner = models.ManyToManyField(
        "practitioners.PractitionerOrganizationPractitionerRoleReference",
        related_name="patient_general_practitioner",
        blank=True,
    )
    managing_organization = models.ForeignKey(
        OrganizationReference,
        on_delete=models.DO_NOTHING,
        null=True,
        blank=True,
        related_name="patient_managing_organization",
    )
    address = models.ManyToManyField(
        Address, related_name="patient_address", blank=True
    )


class PatientReference(BaseReference):
    """Patient reference model."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.CASCADE,
        related_name="patient_reference_identifier",
        null=True,
    )
    patient = models.ForeignKey(
        Patient,
        on_delete=models.CASCADE,
        related_name="patient_reference_patient",
        null=True,
    )


class PatientOrganizationReference(BaseReference):
    """patient organization reference model."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.SET_NULL,
        null=True,
        related_name="patient_organization_reference_identifier",
    )
    patient = models.ForeignKey(
        Patient,
        on_delete=models.SET_NULL,
        related_name="patient_organization_reference_patient",
        null=True,
    )
    organization = models.ForeignKey(
        "organizations.Organization",
        on_delete=models.SET_NULL,
        related_name="patient_organization_reference_organization",
        null=True,
    )


class PatientPractitionerReference(BaseReference):
    """patient practitioner reference model."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.SET_NULL,
        null=True,
        related_name="patient_practitioner_reference_identifier",
    )
    patient = models.ForeignKey(
        Patient,
        on_delete=models.SET_NULL,
        related_name="patient_practitioner_reference_patient",
        null=True,
    )
    practitioner = models.ForeignKey(
        "practitioners.Practitioner",
        on_delete=models.SET_NULL,
        related_name="patient_practitioner_reference_practitioner",
        null=True,
    )



# Generated by Django 5.2 on 2025-04-23 12:46

import django.contrib.postgres.fields
import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('base', '0002_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='CoverageEligibilityResponseError',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('expression', django.contrib.postgres.fields.ArrayField(base_field=models.CharField(max_length=255), null=True, size=None)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='CoverageEligibilityResponseEvent',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('when_date_time', models.DateTimeField(null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='CoverageEligibilityResponseInsurance',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('inforce', models.BooleanField(default=False)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='CoverageEligibilityResponseInsuranceItem',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('excluded', models.BooleanField(default=False)),
                ('name', models.CharField(max_length=255, null=True)),
                ('description', models.TextField(null=True)),
                ('authorization_required', models.BooleanField(default=False)),
                ('authorization_url', models.URLField(null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='CoverageEligibilityResponseInsuranceItemBenefit',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('allowed_string', models.CharField(max_length=255, null=True)),
                ('allowed_unsigned_int', models.PositiveIntegerField(null=True)),
                ('used_string', models.CharField(max_length=255, null=True)),
                ('used_unsigned_int', models.PositiveIntegerField(null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='CoverageEligibilityResponseRequesterReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='CoverageEligibilityResponse',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('status', models.CharField(choices=[('active', 'Active'), ('cancelled', 'Cancelled'), ('draft', 'Draft'), ('entered-in-error', 'Entered in Error')], default='active', max_length=255)),
                ('purpose', django.contrib.postgres.fields.ArrayField(base_field=models.CharField(choices=[('auth-requirements', 'Auth Requirements'), ('benefits', 'Benefits'), ('discovery', 'Discovery'), ('validation', 'Validation')], default='auth-requirements', max_length=255), size=None)),
                ('serviced_date', models.DateField(null=True)),
                ('created', models.DateTimeField(null=True)),
                ('outcome', models.CharField(choices=[('queued', 'Queued'), ('complete', 'Complete'), ('error', 'Error'), ('partial', 'Partial')], default='queued', max_length=255)),
                ('disposition', models.TextField(null=True)),
                ('pre_auth_ref', models.CharField(max_length=255, null=True)),
                ('form', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='coverage_eligibility_response_form', to='base.codeableconcept')),
                ('identifier', models.ManyToManyField(blank=True, related_name='coverage_eligibility_response_identifier', to='base.identifier')),
                ('insurer', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='coverage_eligibility_response_insurer', to='base.organizationreference')),
            ],
            options={
                'abstract': False,
            },
        ),
    ]



# Generated by Django 5.2.1 on 2025-06-30 22:03

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('accounts', '0005_alter_accountcoverage_coverage'),
        ('careteams', '0004_initial'),
        ('encounters', '0004_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='encounter',
            name='account',
            field=models.ManyToManyField(blank=True, related_name='encounter_account', to='accounts.accountreference'),
        ),
        migrations.AlterField(
            model_name='encounter',
            name='care_team',
            field=models.ManyToManyField(blank=True, related_name='encounter_care_team', to='careteams.careteamreference'),
        ),
    ]



"""Substances models."""

from django.db import models

from dfhir.base.models import (
    BaseReference,
    CodeableConcept,
    Identifier,
    Ratio,
    TimeStampedModel,
)

from . import choices


class SubstanceIngredient(TimeStampedModel):
    """Substance Ingredient model."""

    quantity = models.ForeignKey(
        Ratio, related_name="substance_ingredient_quantity", on_delete=models.CASCADE
    )
    substance_codeable_concept = models.ForeignKey(
        CodeableConcept,
        related_name="substance_ingredient_substance",
        on_delete=models.CASCADE,
        null=True,
    )
    substance_reference = models.ForeignKey(
        "SubstanceReference",
        related_name="substance_ingredient_substance_reference",
        on_delete=models.CASCADE,
        null=True,
    )


class Substance(TimeStampedModel):
    """Substance model."""

    identifier = models.ManyToManyField(
        Identifier, related_name="substances_identifier", blank=True
    )
    status = models.CharField(
        max_length=255, null=True, choices=choices.SubstanceStatusChoices.choices
    )
    category = models.ManyToManyField(
        CodeableConcept, related_name="substances_category", blank=True
    )
    # TODO: this should reference substanceDeefintionCodeableReference
    # code = models.ForeignKey(
    #     CodeableConcept, related_name="substances_code", on_delete=models.CASCADE
    # )
    description = models.TextField(null=True)
    expiry = models.DateTimeField(null=True)
    ingredient = models.ManyToManyField(
        SubstanceIngredient,
        related_name="substances_ingredient",
        blank=True,
    )


class SubstanceReference(BaseReference):
    """Substance Reference model."""

    identifier = models.ForeignKey(
        Identifier,
        related_name="substance_reference_identifier",
        on_delete=models.CASCADE,
        null=True,
    )
    substance = models.ForeignKey(
        Substance,
        related_name="substance_reference_substance",
        on_delete=models.CASCADE,
        null=True,
    )


class SubstanceCodeableReference(TimeStampedModel):
    """Substance Codeable Reference model."""

    reference = models.ForeignKey(
        SubstanceReference,
        on_delete=models.CASCADE,
        null=True,
        related_name="substance_codeable_reference_reference",
    )
    concept = models.ForeignKey(
        CodeableConcept,
        on_delete=models.CASCADE,
        null=True,
        related_name="substance_codeable_reference_concept",
    )



"""document reference app."""

from django.apps import AppConfig
from django.utils.translation import gettext_lazy as _


class DocumentreferencesConfig(AppConfig):
    """Document references app configuration."""

    default_auto_field = "django.db.models.BigAutoField"
    name = "dfhir.documentreferences"
    verbose_name = _("Document References")



# Generated by Django 5.2 on 2025-04-23 12:46

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('allergyintolerances', '0005_initial'),
        ('base', '0005_initial'),
        ('conditions', '0004_initial'),
        ('flags', '0003_initial'),
        ('observations', '0002_initial'),
        ('procedures', '0001_initial'),
        ('relatedpersons', '0001_initial'),
        ('riskassessments', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='flagsubjectreference',
            name='procedure',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='flag_subject_reference_procedure', to='procedures.procedure'),
        ),
        migrations.AddField(
            model_name='flagsubjectreference',
            name='related_person',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='flag_subject_reference_related_person', to='relatedpersons.relatedperson'),
        ),
        migrations.AddField(
            model_name='flag',
            name='subject',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='flag_subject', to='flags.flagsubjectreference'),
        ),
        migrations.AddField(
            model_name='flagsupportinginforeference',
            name='allergy_intolerance',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='flag_supporting_info_reference_allergy_intolerance', to='allergyintolerances.allergyintolerance'),
        ),
        migrations.AddField(
            model_name='flagsupportinginforeference',
            name='condition',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='flag_supporting_info_reference_condition', to='conditions.condition'),
        ),
        migrations.AddField(
            model_name='flagsupportinginforeference',
            name='identifier',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='flag_supporting_info_reference_identifier', to='base.identifier'),
        ),
        migrations.AddField(
            model_name='flagsupportinginforeference',
            name='observation',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='flag_supporting_info_reference_observation', to='observations.observation'),
        ),
        migrations.AddField(
            model_name='flagsupportinginforeference',
            name='procedure',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='flag_supporting_info_reference_procedure', to='procedures.procedure'),
        ),
        migrations.AddField(
            model_name='flagsupportinginforeference',
            name='risk_assessment',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='flag_supporting_info_reference_risk_assessment', to='riskassessments.riskassessment'),
        ),
        migrations.AddField(
            model_name='flag',
            name='supporting_info',
            field=models.ManyToManyField(related_name='flag_supports', to='flags.flagsupportinginforeference'),
        ),
    ]



"""Specimen definitions models."""

from django.contrib.postgres.fields import ArrayField
from django.db import models

from dfhir.base.models import (
    BaseReference,
    CodeableConcept,
    Coding,
    ContactDetail,
    Identifier,
    Period,
    Quantity,
    Range,
    TimeStampedModel,
    UsageContext,
)

from . import choices


class SpecimenDefinitionTypeTestedContainerAdditive(TimeStampedModel):
    """Specimen definition type tested container additive."""

    additive_codeable_concept = models.ForeignKey(
        CodeableConcept,
        related_name="specimen_definition_type_tested_container_additive_codeable_concept",
        on_delete=models.CASCADE,
        null=True,
    )
    # additive_reference = models.ForeignKey("substancedefinitions.SubstanceDefinitionReference", on_delete=models.CASCADE, related_name="specimen_definition_type_tested_container_additive_reference", null=True)
    preparation = models.TextField(null=True)


class SpecimenDefinitionTypeTestedContainer(TimeStampedModel):
    """Specimen definition type tested container."""

    material = models.ForeignKey(
        CodeableConcept,
        related_name="specimen_definition_type_tested_container_material",
        on_delete=models.CASCADE,
        null=True,
    )
    type = models.ForeignKey(
        CodeableConcept,
        related_name="specimen_definition_type_tested_container_type",
        on_delete=models.CASCADE,
        null=True,
    )
    cap = models.ForeignKey(
        CodeableConcept,
        related_name="specimen_definition_type_tested_container_cap",
        on_delete=models.CASCADE,
        null=True,
    )
    description = models.TextField(null=True)
    capacity = models.ForeignKey(
        Quantity,
        on_delete=models.CASCADE,
        null=True,
        related_name="specimen_definition_type_tested_container_capacity",
    )
    minimum_volume_quantity = models.ForeignKey(
        Quantity,
        on_delete=models.CASCADE,
        null=True,
        related_name="specimen_definition_type_tested_container_minimum_volume_quantity",
    )
    minimum_volume_string = models.CharField(max_length=255, null=True)
    additive = models.ManyToManyField(
        SpecimenDefinitionTypeTestedContainerAdditive,
        related_name="specimen_definition_type_tested_container_additive",
        blank=True,
    )


class SpecimenDefinitionTypeTestedHandling(TimeStampedModel):
    """Specimen definition type tested handling."""

    temperature_qualifier = models.ForeignKey(
        CodeableConcept,
        on_delete=models.CASCADE,
        related_name="specimen_definition_type_tested_handling_temperature_qualifier",
        null=True,
    )
    temperature_range = models.ForeignKey(
        Range,
        on_delete=models.CASCADE,
        related_name="specimen_definition_type_tested_handling_temperature_range",
        null=True,
    )
    max_duration = models.ForeignKey(
        Quantity,
        on_delete=models.CASCADE,
        related_name="specimen_definition_type_tested_handling_max_duration",
        null=True,
    )
    instruction = models.TextField(null=True)


class SpecimenDefinitionTypeTested(TimeStampedModel):
    """Specimen definition type tested."""

    is_derived = models.BooleanField(null=True)
    type = models.ForeignKey(
        CodeableConcept,
        related_name="specimen_definition_type_tested_type",
        on_delete=models.CASCADE,
        null=True,
    )
    preference = models.CharField(
        max_length=255,
        choices=choices.SpecimenDefinitionTestedTypePreferenceChoices.choices,
        default=choices.SpecimenDefinitionTestedTypePreferenceChoices.PREFERRED,
    )
    container = models.ForeignKey(
        SpecimenDefinitionTypeTestedContainer,
        related_name="specimen_definition_type_tested_container",
        null=True,
        on_delete=models.CASCADE,
    )
    requirements = models.TextField(null=True)
    retention_time = models.ForeignKey(
        Quantity,
        related_name="specimen_definition_type_tested_retention_time",
        on_delete=models.CASCADE,
        null=True,
    )
    single_use = models.BooleanField(null=True)
    retention_criterion = models.ManyToManyField(
        CodeableConcept,
        related_name="specimen_definition_type_tested_retention_criterion",
        blank=True,
    )
    handling = models.ManyToManyField(
        SpecimenDefinitionTypeTestedHandling,
        related_name="specimen_definition_type_tested_handling",
        blank=True,
    )
    testing_destination = models.ManyToManyField(
        CodeableConcept,
        related_name="specimen_definition_type_tested_testing_destination",
        blank=True,
    )


class SpecimenDefinition(TimeStampedModel):
    """Specimen Definition model."""

    uri = models.URLField(null=True)
    identifier = models.ForeignKey(
        Identifier,
        related_name="specimen_definitions_identifier",
        null=True,
        on_delete=models.CASCADE,
    )
    version = models.CharField(max_length=255, null=True)
    version_algorithm_string = models.CharField(max_length=255, null=True)
    version_algorithm_coding = models.ForeignKey(
        Coding,
        related_name="specimen_definition_version_algorithm_coding",
        null=True,
        on_delete=models.CASCADE,
    )
    name = models.CharField(max_length=255, null=True)
    title = models.CharField(max_length=255, null=True)
    # derived_from_canonical =
    derived_from_uri = ArrayField(models.URLField(), null=True)
    status = models.CharField(
        max_length=255,
        choices=choices.SpecimenDefinitionStatusChoices.choices,
        default=choices.SpecimenDefinitionStatusChoices.ACTIVE,
    )
    experimental = models.BooleanField(null=True)
    subject_codeable_concept = models.ForeignKey(
        CodeableConcept,
        related_name="specimen_definition_subject_codeable_concept",
        null=True,
        on_delete=models.CASCADE,
    )
    subject_reference = models.ForeignKey(
        "groups.GroupReference",
        related_name="specimen_definition_subject_reference",
        null=True,
        on_delete=models.CASCADE,
    )
    date = models.DateField(null=True)
    publisher = models.CharField(max_length=255, null=True)
    contact = models.ManyToManyField(
        ContactDetail, related_name="specimen_definition_contact", blank=True
    )
    description = models.TextField(null=True)
    use_context = models.ManyToManyField(
        UsageContext, related_name="specimen_definition_use_context", blank=True
    )
    jurisdiction = models.ManyToManyField(
        CodeableConcept, related_name="specimen_definition_jurisdiction", blank=True
    )
    purpose = models.TextField(null=True)
    copyright = models.TextField(null=True)
    copyright_label = models.CharField(max_length=255, null=True)
    approval_date = models.DateField(null=True)
    last_review_date = models.DateField(null=True)
    effective_period = models.ForeignKey(
        Period,
        related_name="specimen_definition_effective_period",
        on_delete=models.CASCADE,
        null=True,
    )
    type_collected = models.ForeignKey(
        CodeableConcept,
        related_name="specimen_definition_type_collected",
        on_delete=models.CASCADE,
        null=True,
    )
    patient_preparation = models.ManyToManyField(
        CodeableConcept,
        related_name="specimen_definition_patient_preparation",
        blank=True,
    )
    time_aspect = models.CharField(max_length=255, null=True)
    collection = models.ManyToManyField(
        CodeableConcept, related_name="specimen_definition_collection", blank=True
    )
    type_tested = models.ManyToManyField(
        SpecimenDefinitionTypeTested,
        related_name="specimen_definition_type_tested",
        blank=True,
    )


class SpecimenDefinitionReference(BaseReference):
    """Specimen Definition Reference."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.CASCADE,
        null=True,
        related_name="specimen_definition_reference_identifier",
    )
    specimen_definition = models.ForeignKey(
        SpecimenDefinition,
        on_delete=models.CASCADE,
        null=True,
        related_name="specimen_definition_reference_specimen_definition",
    )



"""Healthcareservices app configuration."""

from django.apps import AppConfig
from django.utils.translation import gettext_lazy as _


class HealthcareservicesConfig(AppConfig):
    """Healthcareservices app configuration."""

    default_auto_field = "django.db.models.BigAutoField"
    name = "dfhir.healthcareservices"
    verbose_name = _("Healthcareservices")



"""medication dispenses models."""

from django.db import models

from dfhir.base.models import (
    BaseReference,
    CodeableConcept,
    Identifier,
    TimeStampedModel,
)
from dfhir.medicationdispenses.choices import MedicationDispenseStatus


class MedicationDispensePerformerActorReference(BaseReference):
    """medication dispense performer actor reference model."""

    identifier = models.ForeignKey(Identifier, on_delete=models.SET_NULL, null=True)
    practitioner = models.ForeignKey(
        "practitioners.Practitioner", on_delete=models.SET_NULL, null=True
    )
    practitioner_role = models.ForeignKey(
        "practitionerroles.PractitionerRole", on_delete=models.SET_NULL, null=True
    )
    organization = models.ForeignKey(
        "organizations.Organization", on_delete=models.SET_NULL, null=True
    )
    patient = models.ForeignKey(
        "patients.Patient", on_delete=models.SET_NULL, null=True
    )
    device = models.ForeignKey("devices.Device", on_delete=models.SET_NULL, null=True)
    related_person = models.ForeignKey(
        "relatedpersons.RelatedPerson", on_delete=models.SET_NULL, null=True
    )
    care_team = models.ForeignKey(
        "careteams.CareTeam", on_delete=models.SET_NULL, null=True
    )


class MedicationDispenseReceiverReference(BaseReference):
    """medication dispense receiver reference model."""

    identifier = models.ForeignKey(Identifier, on_delete=models.SET_NULL, null=True)
    patient = models.ForeignKey(
        "patients.Patient", on_delete=models.SET_NULL, null=True
    )
    practitioner = models.ForeignKey(
        "practitioners.Practitioner", on_delete=models.SET_NULL, null=True
    )
    related_person = models.ForeignKey(
        "relatedpersons.RelatedPerson", on_delete=models.SET_NULL, null=True
    )
    location = models.ForeignKey(
        "locations.Location", on_delete=models.SET_NULL, null=True
    )
    practitioner_role = models.ForeignKey(
        "practitionerroles.PractitionerRole", on_delete=models.SET_NULL, null=True
    )


class MedicationDispensePerformer(TimeStampedModel):
    """medication dispense performer model."""

    function = models.ForeignKey(
        "base.CodeableConcept",
        on_delete=models.SET_NULL,
        null=True,
        related_name="medication_dispense_performer_function",
    )
    actor = models.ForeignKey(
        MedicationDispensePerformerActorReference,
        on_delete=models.SET_NULL,
        null=True,
        related_name="medication_dispense_performer_actor",
    )


class MedicationDispenseSubstitutionResponsiblePartyReference(BaseReference):
    """medication dispense substitution responsibility party."""

    identifier = models.ForeignKey(Identifier, on_delete=models.SET_NULL, null=True)
    practitioner = models.ForeignKey(
        "practitioners.Practitioner", on_delete=models.SET_NULL, null=True
    )
    practitioner_role = models.ForeignKey(
        "practitionerroles.PractitionerRole", on_delete=models.SET_NULL, null=True
    )
    organization = models.ForeignKey(
        "organizations.Organization", on_delete=models.SET_NULL, null=True
    )


class MedicationDispenseSubstitution(TimeStampedModel):
    """mediation dispense substitution model."""

    was_substituted = models.BooleanField(default=False)
    type = models.ForeignKey(
        CodeableConcept,
        on_delete=models.SET_NULL,
        null=True,
        related_name="medication_dispense_substitution_type",
    )
    reason = models.ManyToManyField(
        CodeableConcept,
        blank=True,
        related_name="medication_dispense_substitution_reason",
    )
    responsible_party = models.ManyToManyField(
        MedicationDispenseSubstitutionResponsiblePartyReference,
        blank=True,
        related_name="medication_dispense_substitution_responsible_party",
    )


class MedicationDispense(TimeStampedModel):
    """medication dispense model."""

    identifier = models.ManyToManyField(
        Identifier, blank=True, related_name="medication_dispense_identifier"
    )

    based_on = models.ManyToManyField("careplans.CarePlanReference", blank=True)

    part_of = models.ManyToManyField(
        "procedures.ProcedureReference",
        blank=True,
        related_name="medication_dispense_part_of",
    )
    status = models.CharField(
        max_length=255, null=True, choices=MedicationDispenseStatus.choices
    )

    not_performed_reason = models.ForeignKey(
        "detectedissues.DetectedIssueCodeableReference",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="medication_dispense_not_performed_reason",
    )
    status_changed = models.DateTimeField(null=True)
    category = models.ManyToManyField(
        CodeableConcept, blank=True, related_name="medication_dispense_category"
    )

    medication = models.ForeignKey(
        "medications.MedicationCodeableReference",
        on_delete=models.CASCADE,
        null=True,
        related_name="medication_dispense_medication",
    )

    subject = models.ForeignKey(
        "patients.PatientGroupReference",
        on_delete=models.SET_NULL,
        null=True,
        related_name="medication_dispense_subject",
    )

    encounter = models.ForeignKey(
        "encounters.EncounterReference",
        on_delete=models.SET_NULL,
        null=True,
        related_name="medication_dispense_encounter",
    )
    supporting_information = models.ManyToManyField(
        "base.Reference",
        blank=True,
        related_name="medication_dispense_supporting_information",
    )

    performer = models.ManyToManyField(
        MedicationDispensePerformer,
        blank=True,
        related_name="medication_dispense_performer",
    )

    location = models.ForeignKey(
        "locations.LocationReference",
        on_delete=models.SET_NULL,
        null=True,
        related_name="medication_dispense_location",
    )

    authorizing_prescription = models.ManyToManyField(
        "medicationrequests.MedicationRequestReference",
        blank=True,
        related_name="medication_dispense_authorizing_prescription",
    )

    type = models.ForeignKey(
        CodeableConcept,
        on_delete=models.SET_NULL,
        null=True,
        related_name="medication_dispense_type",
    )

    quantity = models.ForeignKey(
        "base.Quantity",
        on_delete=models.SET_NULL,
        null=True,
        related_name="medication_dispense_quantity",
    )
    days_supply = models.ForeignKey(
        "base.Quantity",
        on_delete=models.SET_NULL,
        null=True,
        related_name="medication_dispense_days_supply",
    )
    recorded = models.DateTimeField(null=True)
    when_prepared = models.DateTimeField(null=True)
    when_handed_over = models.DateTimeField(null=True)
    destination = models.ForeignKey(
        "locations.LocationReference",
        on_delete=models.SET_NULL,
        null=True,
        related_name="medication_dispense_destination",
    )
    receiver = models.ManyToManyField(
        MedicationDispenseReceiverReference,
        blank=True,
        related_name="medication_dispense_receiver",
    )
    note = models.ManyToManyField(
        "base.Annotation", blank=True, related_name="medication_dispense_note"
    )
    rendered_dosage_instructions = models.TextField(null=True)
    # TODO: dosage_instruction = models.ManyToManyField("Dosage", blank=True)
    substitution = models.ForeignKey(
        MedicationDispenseSubstitution,
        on_delete=models.SET_NULL,
        null=True,
        related_name="medication_dispense_substitution",
    )
    event_history = models.ManyToManyField(
        "provenances.Provenance",
        blank=True,
        related_name="medication_dispense_event_history",
    )



# Generated by Django 5.2 on 2025-05-05 14:45

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('documentreferences', '0005_initial'),
        ('riskassessments', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='riskassessmentreasonreference',
            name='document_reference',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='risk_assessment_reason_references_document_reference', to='documentreferences.documentreference'),
        ),
    ]



"""Plandefinitions app views."""

from django.http import Http404
from drf_spectacular.utils import extend_schema
from rest_framework import status
from rest_framework.permissions import AllowAny
from rest_framework.response import Response
from rest_framework.views import APIView

from .models import PlanDefinition
from .serializers import PlanDefinitionSerializer


class PlanDefinitionListView(APIView):
    """PlanDefinition list view."""

    permission_classes = [AllowAny]

    @extend_schema(responses={200: PlanDefinitionSerializer(many=True)})
    def get(self, request):
        """Get a list of plan definitions."""
        plan_definitions = PlanDefinition.objects.all()
        serializer = PlanDefinitionSerializer(plan_definitions, many=True)
        return Response(serializer.data, status=status.HTTP_200_OK)

    @extend_schema(
        request=PlanDefinitionSerializer, responses={201: PlanDefinitionSerializer}
    )
    def post(self, request):
        """Create a plan definition."""
        serializer = PlanDefinitionSerializer(data=request.data)
        serializer.is_valid(raise_exception=True)
        serializer.save()
        return Response(serializer.data, status=status.HTTP_201_CREATED)


class PlanDefinitionDetailView(APIView):
    """PlanDefinition detail view."""

    permission_classes = [AllowAny]

    def get_object(self, pk):
        """Get a plan definition object."""
        try:
            return PlanDefinition.objects.get(pk=pk)
        except PlanDefinition.DoesNotExist as err:
            raise Http404 from err

    @extend_schema(responses={200: PlanDefinitionSerializer})
    def get(self, request, pk=None):
        """Get a plan definition."""
        plan_definition = self.get_object(pk)
        serializer = PlanDefinitionSerializer(plan_definition)
        return Response(serializer.data, status=status.HTTP_200_OK)

    @extend_schema(
        request=PlanDefinitionSerializer, responses={200: PlanDefinitionSerializer}
    )
    def patch(self, request, pk=None):
        """Update a plan definition."""
        plan_definition = self.get_object(pk)
        serializer = PlanDefinitionSerializer(
            plan_definition, data=request.data, partial=True
        )
        serializer.is_valid(raise_exception=True)
        serializer.save()
        return Response(serializer.data, status=status.HTTP_200_OK)

    @extend_schema(responses={204: None})
    def delete(self, request, pk=None):
        """Delete a plan definition."""
        plan_definition = self.get_object(pk)
        plan_definition.delete()
        return Response(status=status.HTTP_204_NO_CONTENT)



"""immunization recommendations app."""

from django.apps import AppConfig
from django.utils.translation import gettext_lazy as _


class ImmunizationrecommendationsConfig(AppConfig):
    """app config."""

    default_auto_field = "django.db.models.BigAutoField"
    name = "dfhir.immunizationrecommendations"
    verbose_name = _("Immunization Recommendations")



# Generated by Django 5.2 on 2025-04-23 12:46

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('base', '0004_initial'),
        ('biologicallyderivedproducts', '0001_initial'),
        ('contracts', '0002_initial'),
        ('devices', '0001_initial'),
        ('inventoryitems', '0002_initial'),
        ('locations', '0001_initial'),
        ('medications', '0001_initial'),
        ('nutritionproducts', '0001_initial'),
        ('organizations', '0001_initial'),
        ('patients', '0001_initial'),
        ('practitionerroles', '0001_initial'),
        ('practitioners', '0001_initial'),
        ('substances', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='SupplyDelivery',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('status', models.CharField(choices=[('in-progress', 'In Progress'), ('completed', 'Completed'), ('abandoned', 'Abandoned'), ('entered-in-error', 'Entered In Error')], max_length=255, null=True)),
                ('occurrence_date_time', models.DateTimeField(blank=True, null=True)),
                ('identifier', models.ManyToManyField(blank=True, related_name='supply_delivery_identifier', to='base.identifier')),
                ('occurrence_period', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='supply_delivery_occurrence_period', to='base.period')),
                ('occurrence_timing', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='supply_delivery_occurrence', to='base.timing')),
                ('patient', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='supply_delivery_patient', to='patients.patientreference')),
                ('stage', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='supply_delivery_stage', to='base.codeableconcept')),
                ('type', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='supply_delivery_type', to='base.codeableconcept')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='SupplyDeliverReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
                ('identifier', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='supply_delivery_reference_identifier', to='base.identifier')),
                ('supply_delivery', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='supply_delivery_reference_contract', to='supplydeliveries.supplydelivery')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='SupplyDeliveryContractReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
                ('contract', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='supply_delivery_contracts_reference_contract', to='contracts.contract')),
                ('identifier', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='supply_delivery_contracts_reference_identifier', to='base.identifier')),
                ('supply_delivery', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='supply_delivery_contracts_supply_delivery', to='supplydeliveries.supplydelivery')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.AddField(
            model_name='supplydelivery',
            name='part_of',
            field=models.ManyToManyField(blank=True, related_name='supply_delivery_part_of', to='supplydeliveries.supplydeliverycontractreference'),
        ),
        migrations.CreateModel(
            name='SupplyDeliveryDestinationReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
                ('identifier', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='supply_delivery_destination_references_identifier', to='base.identifier')),
                ('location', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='supply_delivery_destination_references_location', to='locations.location')),
                ('organization', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='supply_delivery_destination_references_organization', to='organizations.organization')),
                ('practitioner', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='supply_delivery_destination_references_practitioner', to='practitioners.practitioner')),
                ('practitioner_role', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='supply_delivery_destination_references_practitioner_role', to='practitionerroles.practitionerrole')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.AddField(
            model_name='supplydelivery',
            name='destination',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='supply_delivery_destination', to='supplydeliveries.supplydeliverydestinationreference'),
        ),
        migrations.CreateModel(
            name='SupplyDeliveryReceiverReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
                ('identifier', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='supply_delivery_receiver_references_identifier', to='base.identifier')),
                ('organization', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='supply_delivery_receiver_references_organization', to='organizations.organization')),
                ('practitioner', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='supply_delivery_receiver_references_practitioner', to='practitioners.practitioner')),
                ('practitioner_role', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='supply_delivery_receiver_references_practitioner_role', to='practitionerroles.practitionerrole')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.AddField(
            model_name='supplydelivery',
            name='receiver',
            field=models.ManyToManyField(blank=True, related_name='supply_delivery_receiver', to='supplydeliveries.supplydeliveryreceiverreference'),
        ),
        migrations.CreateModel(
            name='SupplyDeliverySuppliedItem',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('condition', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='supply_delivery_supplied_item_condition', to='base.codeableconcept')),
                ('item_codeable_concept', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='supply_delivery_supplied_item_item_codealbe_concept', to='base.codeableconcept')),
                ('quantity', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='supply_delivery_supplied_item_quantity', to='base.simplequantity')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.AddField(
            model_name='supplydelivery',
            name='supplied_item',
            field=models.ManyToManyField(blank=True, related_name='supply_delivery_supplied_item', to='supplydeliveries.supplydeliverysupplieditem'),
        ),
        migrations.CreateModel(
            name='SupplyDeliverySuppliedItemItemReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
                ('biologically_derived_product', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='supply_delivery_supplied_item_item_references_biologically_derived_product', to='biologicallyderivedproducts.biologicallyderivedproduct')),
                ('device', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='supply_delivery_supplied_item_item_references_device', to='devices.device')),
                ('identifier', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='supply_delivery_supplied_item_item_references_identifier', to='base.identifier')),
                ('inventory_item', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='supply_delivery_supplied_item_item_references_inventory_item', to='inventoryitems.inventoryitem')),
                ('medication', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='supply_delivery_supplied_item_item_references_medication', to='medications.medication')),
                ('nutrition_product', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='supply_delivery_supplied_item_item_references_nutrition_product', to='nutritionproducts.nutritionproduct')),
                ('sustance', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='supply_delivery_supplied_item_item_references_substance', to='substances.substance')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.AddField(
            model_name='supplydeliverysupplieditem',
            name='item_reference',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='supply_delivery_supplied_item_item_reference', to='supplydeliveries.supplydeliverysupplieditemitemreference'),
        ),
        migrations.CreateModel(
            name='SupplyDeliverySupplierReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
                ('identifier', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='supply_delivery_supplier_references_identifier', to='base.identifier')),
                ('organization', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='supply_delivery_supplier_references_organization', to='organizations.organization')),
                ('practitioner', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='supply_delivery_supplier_references_practitioner', to='practitioners.practitioner')),
                ('practitioner_role', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='supply_delivery_supplier_references_practitioner_role', to='practitionerroles.practitionerrole')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.AddField(
            model_name='supplydelivery',
            name='supplier',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='supply_delivery_supplier', to='supplydeliveries.supplydeliverysupplierreference'),
        ),
    ]



# Generated by Django 5.2 on 2025-04-23 12:46

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('appointments', '0004_initial'),
        ('base', '0005_initial'),
        ('documentreferences', '0004_initial'),
        ('encounters', '0003_initial'),
        ('episodeofcares', '0002_initial'),
        ('servicerequests', '0001_initial'),
        ('supplyrequests', '0001_initial'),
        ('visionprescriptions', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='documentreferencebasedonreference',
            name='service_request',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='document_reference_based_on_reference_service_request', to='servicerequests.servicerequest'),
        ),
        migrations.AddField(
            model_name='documentreferencebasedonreference',
            name='supply_request',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='document_reference_based_on_reference_supply_request', to='supplyrequests.supplyrequest'),
        ),
        migrations.AddField(
            model_name='documentreferencebasedonreference',
            name='vision_prescription',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='document_reference_based_on_reference_vision_prescription', to='visionprescriptions.visionprescription'),
        ),
        migrations.AddField(
            model_name='documentreference',
            name='based_on',
            field=models.ManyToManyField(blank=True, related_name='document_reference_based_on', to='documentreferences.documentreferencebasedonreference'),
        ),
        migrations.AddField(
            model_name='documentreferencecodeablereference',
            name='concept',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='document_reference_codeable_references_concept', to='base.codeableconcept'),
        ),
        migrations.AddField(
            model_name='documentreferencecodeablereference',
            name='reference',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='document_reference_codeable_references_reference', to='base.reference'),
        ),
        migrations.AddField(
            model_name='documentreferencecontent',
            name='attachment',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='document_reference_contents_attachment', to='base.attachment'),
        ),
        migrations.AddField(
            model_name='documentreference',
            name='content',
            field=models.ManyToManyField(blank=True, related_name='document_reference_contents', to='documentreferences.documentreferencecontent'),
        ),
        migrations.AddField(
            model_name='documentreferencecontentprofile',
            name='value_coding',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='document_reference_content_profiles_value_coding', to='base.coding'),
        ),
        migrations.AddField(
            model_name='documentreferencecontent',
            name='profile',
            field=models.ManyToManyField(blank=True, related_name='document_reference_contents_profile', to='documentreferences.documentreferencecontentprofile'),
        ),
        migrations.AddField(
            model_name='documentreferencecontextreference',
            name='appointment',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='document_reference_context_reference_appointment', to='appointments.appointment'),
        ),
        migrations.AddField(
            model_name='documentreferencecontextreference',
            name='encounter',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='document_reference_context_reference_encounter', to='encounters.encounter'),
        ),
        migrations.AddField(
            model_name='documentreferencecontextreference',
            name='episode_of_care',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='document_reference_context_reference_episode_of_care', to='episodeofcares.episodeofcare'),
        ),
        migrations.AddField(
            model_name='documentreferencecontextreference',
            name='identifier',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='document_reference_context_references_identifier', to='base.identifier'),
        ),
        migrations.AddField(
            model_name='documentreference',
            name='context',
            field=models.ManyToManyField(blank=True, related_name='document_reference_contexts', to='documentreferences.documentreferencecontextreference'),
        ),
        migrations.AddField(
            model_name='documentreferencereference',
            name='document_reference',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='document_reference_references_document_reference', to='documentreferences.documentreference'),
        ),
        migrations.AddField(
            model_name='documentreferencereference',
            name='identifier',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='document_reference_references_identifier', to='base.identifier'),
        ),
        migrations.AddField(
            model_name='documentreferencerelatesto',
            name='code',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='document_reference_relates_tos_code', to='base.codeableconcept'),
        ),
        migrations.AddField(
            model_name='documentreferencerelatesto',
            name='target',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='document_reference_relates_tos_target', to='documentreferences.documentreferencereference'),
        ),
        migrations.AddField(
            model_name='documentreference',
            name='relates_to',
            field=models.ManyToManyField(blank=True, related_name='document_reference_relates_tos', to='documentreferences.documentreferencerelatesto'),
        ),
    ]



# Generated by Django 5.2 on 2025-04-23 12:46

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('appointments', '0001_initial'),
        ('base', '0001_initial'),
        ('healthcareservices', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='appointment',
            name='appointment_type',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='appointment_type', to='base.codeableconcept'),
        ),
        migrations.AddField(
            model_name='appointment',
            name='cancellation_reason',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, to='base.codeableconcept'),
        ),
        migrations.AddField(
            model_name='appointment',
            name='identifier',
            field=models.ManyToManyField(blank=True, related_name='appointment_identifier', to='base.identifier'),
        ),
        migrations.AddField(
            model_name='appointment',
            name='klass',
            field=models.ManyToManyField(blank=True, related_name='appointment_class', to='base.codeableconcept'),
        ),
        migrations.AddField(
            model_name='appointment',
            name='note',
            field=models.ManyToManyField(blank=True, related_name='appointment_note', to='base.annotation'),
        ),
        migrations.AddField(
            model_name='appointment',
            name='priority',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='appointment_priority', to='base.codeableconcept'),
        ),
        migrations.AddField(
            model_name='appointment',
            name='requested_period',
            field=models.ManyToManyField(blank=True, related_name='appointment_requested_period', to='base.period'),
        ),
        migrations.AddField(
            model_name='appointment',
            name='service_category',
            field=models.ManyToManyField(blank=True, related_name='service_category', to='base.codeableconcept'),
        ),
        migrations.AddField(
            model_name='appointment',
            name='service_type',
            field=models.ManyToManyField(blank=True, related_name='appointment_service_type', to='healthcareservices.healthcareservicecodeablereference'),
        ),
    ]



# Generated by Django 5.2 on 2025-04-23 12:46

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('adverseevents', '0001_initial'),
        ('base', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='adverseevent',
            name='category',
            field=models.ManyToManyField(blank=True, related_name='adverse_event_category', to='base.codeableconcept'),
        ),
        migrations.AddField(
            model_name='adverseevent',
            name='cause_period',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='adverse_event_cause_period', to='base.period'),
        ),
        migrations.AddField(
            model_name='adverseevent',
            name='code',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='adverse_event_code', to='base.codeableconcept'),
        ),
        migrations.AddField(
            model_name='adverseevent',
            name='effect_period',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='adverse_event_effect_period', to='base.period'),
        ),
    ]



"""Plandefinitions serializers."""

from drf_writable_nested import WritableNestedModelSerializer

from dfhir.base.serializers import (
    AgeSerializer,
    BaseReferenceModelSerializer,
    BaseWritableNestedModelSerializer,
    CodeableConceptSerializer,
    CodingSerializer,
    ContactDetailSerializer,
    DurationSerializer,
    ExpressionSerializer,
    IdentifierSerializer,
    PeriodSerializer,
    QuantitySerializer,
    RangeSerializer,
    RatioSerializer,
    RelatedArtifactSerializer,
    TimingSerializer,
    TriggerDefinitionSerializer,
    UsageContextSerializer,
)
from dfhir.groups.serializers import GroupReferenceSerializer
from dfhir.locations.serializers import LocationCodeableReferenceSerializer

from .models import (
    PlanDefinition,
    PlanDefinitionAction,
    PlanDefinitionActionCondition,
    PlanDefinitionActionDynamicValue,
    PlanDefinitionActionInput,
    PlanDefinitionActionOutput,
    PlanDefinitionActionParticipant,
    PlanDefinitionActionRelatedAction,
    PlanDefinitionActor,
    PlanDefinitionActorOptionTypeReferenceReference,
    PlanDefinitionGoal,
    PlanDefinitionGoalTarget,
    PlanDefinitionSubjectReference,
    PlanDefintionActorOption,
)


class PlanDefinitionSubjectReferenceSerializer(BaseReferenceModelSerializer):
    """PlanDefinitionSubjectReferenceSerializer."""

    identifier = IdentifierSerializer(required=False)

    class Meta:
        """Meta class."""

        model = PlanDefinitionSubjectReference
        exclude = ["created_at", "updated_at"]


class PlanDefinitionActorOptionTypeReferenceReferenceSerializer(
    BaseReferenceModelSerializer
):
    """PlanDefinitionActorOptionTypeReferenceReferenceSerializer."""

    identifier = IdentifierSerializer(required=False)

    class Meta:
        """Meta class."""

        model = PlanDefinitionActorOptionTypeReferenceReference
        exclude = ["created_at", "updated_at"]


class PlanDefinitionActorOptionSerializer(WritableNestedModelSerializer):
    """PlanDefinitionActorOptionSerializer."""

    type_reference = PlanDefinitionActorOptionTypeReferenceReferenceSerializer(
        many=False, required=False
    )
    role = CodeableConceptSerializer(many=False, required=False)

    class Meta:
        """Meta class."""

        model = PlanDefintionActorOption
        exclude = ["created_at", "updated_at"]


class PlanDefinitionActionConditionSerializer(WritableNestedModelSerializer):
    """PlanDefinitionActionConditionSerializer."""

    expression = ExpressionSerializer(many=False, required=False)

    class Meta:
        """Meta class."""

        model = PlanDefinitionActionCondition
        exclude = ["created_at", "updated_at"]


class PlanDefinitionActionInputSerializer(WritableNestedModelSerializer):
    """PlanDefinitionActionInputSerializer."""

    class Meta:
        """Meta class."""

        model = PlanDefinitionActionInput
        exclude = ["created_at", "updated_at"]


class PlanDefinitionActionOutputSerializer(WritableNestedModelSerializer):
    """PlanDefinitionActionOutputSerializer."""

    class Meta:
        """Meta class."""

        model = PlanDefinitionActionOutput
        exclude = ["created_at", "updated_at"]


class PlanDefinitionActionParticipantSerializer(WritableNestedModelSerializer):
    """PlanDefinitionActionParticipantSerializer."""

    type_reference = PlanDefinitionActorOptionTypeReferenceReferenceSerializer(
        many=False, required=False
    )
    role = CodeableConceptSerializer(many=False, required=False)
    function = CodeableConceptSerializer(many=False, required=False)

    class Meta:
        """Meta class."""

        model = PlanDefinitionActionParticipant
        exclude = ["created_at", "updated_at"]


class PlanDefinitionActionDynamicValueSerializer(WritableNestedModelSerializer):
    """PlanDefinitionActionDynamicValueSerializer."""

    expression = ExpressionSerializer(many=False, required=False)

    class Meta:
        """Meta class."""

        model = PlanDefinitionActionDynamicValue
        exclude = ["created_at", "updated_at"]


class PlanDefinitionActionRelatedActionSerializer(WritableNestedModelSerializer):
    """PlanDefinitionActionRelatedActionSerializer."""

    offset_duration = DurationSerializer(many=False, required=False)
    offset_range = RangeSerializer(many=False, required=False)

    class Meta:
        """Meta class."""

        model = PlanDefinitionActionRelatedAction
        exclude = ["created_at", "updated_at"]


class PlanDefinitionActionSerializer(WritableNestedModelSerializer):
    """PlanDefinitionActionSerializer."""

    code = CodeableConceptSerializer(many=False, required=False)
    reason = CodeableConceptSerializer(many=True, required=False)
    documentation = RelatedArtifactSerializer(many=True, required=False)
    subject_codeable_concept = CodeableConceptSerializer(many=False, required=False)
    subject_reference = GroupReferenceSerializer(many=False, required=False)
    trigger = TriggerDefinitionSerializer(many=True, required=False)
    condition = PlanDefinitionActionConditionSerializer(many=True, required=False)
    input = PlanDefinitionActionInputSerializer(many=True, required=False)
    output = PlanDefinitionActionOutputSerializer(many=True, required=False)
    related_action = PlanDefinitionActionRelatedActionSerializer(
        many=True, required=False
    )
    timing_age = AgeSerializer(many=False, required=False)
    timing_duration = DurationSerializer(many=False, required=False)
    timing_range = RangeSerializer(many=False, required=False)
    timing_timing = TimingSerializer(many=False, required=False)
    location = LocationCodeableReferenceSerializer(many=False, required=False)
    participant = PlanDefinitionActionParticipantSerializer(many=True, required=False)
    type = CodeableConceptSerializer(many=False, required=False)
    dynamic_value = PlanDefinitionActionDynamicValueSerializer(
        many=True, required=False
    )

    class Meta:
        """Meta class."""

        model = PlanDefinitionAction
        exclude = ["created_at", "updated_at"]


class PlanDefinitionActorSerializer(WritableNestedModelSerializer):
    """PlanDefinitionActorSerializer."""

    option = PlanDefinitionActorOptionSerializer(many=True, required=False)

    class Meta:
        """Meta class."""

        model = PlanDefinitionActor
        exclude = ["created_at", "updated_at"]


class PlanDefinitionGoalTargetSerializer(WritableNestedModelSerializer):
    """PlanDefinitionGoalTargetSerializer."""

    measure = CodeableConceptSerializer(many=False, required=False)
    detail_quantity = QuantitySerializer(many=False, required=False)
    detail_range = RangeSerializer(many=False, required=False)
    detail_codeable_concept = CodeableConceptSerializer(many=False, required=False)
    detail_ratio = RatioSerializer(many=False, required=False)
    due = PeriodSerializer(many=False, required=False)

    class Meta:
        """Meta class."""

        model = PlanDefinitionGoalTarget
        exclude = ["created_at", "updated_at"]


class PlanDefinitionGoalSerializer(WritableNestedModelSerializer):
    """PlanDefinitionGoalSerializer."""

    category = CodeableConceptSerializer(many=False, required=False)
    description = CodeableConceptSerializer(many=False, required=False)
    priority = CodeableConceptSerializer(many=False, required=False)
    start = CodeableConceptSerializer(many=False, required=False)
    addresses = CodeableConceptSerializer(many=True, required=False)
    documentation = RelatedArtifactSerializer(many=True, required=False)
    target = PlanDefinitionGoalTargetSerializer(many=True, required=False)

    class Meta:
        """Meta class."""

        model = PlanDefinitionGoal
        exclude = ["created_at", "updated_at"]


class PlanDefinitionSerializer(BaseWritableNestedModelSerializer):
    """PlanDefinitionSerializer."""

    identifier = IdentifierSerializer(many=True, required=False)
    version_algorithm_coding = CodingSerializer(many=False, required=False)
    type = CodeableConceptSerializer(many=False, required=False)
    subject_codeable_concept = CodeableConceptSerializer(many=False, required=False)
    subject_reference = PlanDefinitionSubjectReferenceSerializer(
        many=False, required=False
    )
    contact = ContactDetailSerializer(many=True, required=False)
    use_context = UsageContextSerializer(many=True, required=False)
    jurisdiction = CodeableConceptSerializer(many=True, required=False)
    effective_period = PeriodSerializer(many=False, required=False)
    topic = CodeableConceptSerializer(many=True, required=False)
    author = ContactDetailSerializer(many=True, required=False)
    editor = ContactDetailSerializer(many=True, required=False)
    reviewer = ContactDetailSerializer(many=True, required=False)
    endorser = ContactDetailSerializer(many=True, required=False)
    related_artifact = RelatedArtifactSerializer(many=True, required=False)
    goal = PlanDefinitionGoalSerializer(many=True, required=False)
    actor = PlanDefinitionActorSerializer(many=True, required=False)
    action = PlanDefinitionActionSerializer(many=True, required=False)
    as_needed_codeable_concept = CodeableConceptSerializer(many=False, required=False)

    class Meta:
        """Meta class."""

        model = PlanDefinition
        exclude = ["created_at", "updated_at"]



"""Location serializers."""

from drf_writable_nested.serializers import WritableNestedModelSerializer
from rest_framework.serializers import ModelSerializer

from dfhir.base.serializers import (
    AddressSerializer,
    AvailabilitySerializer,
    BaseReferenceModelSerializer,
    BaseWritableNestedModelSerializer,
    CodeableConceptSerializer,
    CodingSerializer,
    ExtendedContactDetailSerializer,
    IdentifierSerializer,
    OrganizationReferenceSerializer,
    VirtualServiceDetailsSerializer,
)
from dfhir.endpoints.serializers import EndpointReferenceSerializer

from .models import (
    Location,
    LocationCodeableReference,
    LocationOrganizationReference,
    LocationReference,
    Position,
)


class LocationReferenceSerializer(BaseReferenceModelSerializer):
    """Location reference serializer."""

    identifier = IdentifierSerializer(many=False, required=False)

    class Meta:
        """Meta class."""

        model = LocationReference
        exclude = ["created_at", "updated_at"]


class LocationOrganizationReferenceSerializer(BaseReferenceModelSerializer):
    """Location organization reference serializer."""

    identifier = IdentifierSerializer(many=False, required=False)

    class Meta:
        """Meta class."""

        model = LocationOrganizationReference
        exclude = ["created_at", "updated_at"]


class PositionSerializer(ModelSerializer):
    """Position serializer."""

    class Meta:
        """Meta class."""

        model = Position
        exclude = ["created_at", "updated_at"]


class LocationSerializer(BaseWritableNestedModelSerializer):
    """Location serializer."""

    identifier = IdentifierSerializer(many=True, required=False)
    position = PositionSerializer(many=False, required=False)
    operational_status = CodingSerializer(many=False, required=False)
    contact = ExtendedContactDetailSerializer(many=True, required=False)
    address = AddressSerializer(many=False, required=False)
    hours_of_operation = AvailabilitySerializer(many=False, required=False)
    form = CodeableConceptSerializer(many=False, required=False)
    type = CodeableConceptSerializer(many=True, required=False)
    managing_organization = OrganizationReferenceSerializer(many=False, required=False)
    characteristic = CodeableConceptSerializer(many=True, required=False)
    virtual_service = VirtualServiceDetailsSerializer(many=True, required=False)
    endpoint = EndpointReferenceSerializer(many=True, required=False)
    part_of = LocationReferenceSerializer(many=False, required=False)

    class Meta:
        """Meta class."""

        model = Location
        exclude = ["created_at", "updated_at"]


class LocationCodeableReferenceSerializer(WritableNestedModelSerializer):
    """Location codeable reference serializer."""

    concept = CodeableConceptSerializer(many=False, required=False)
    reference = LocationReferenceSerializer(many=False, required=False)

    class Meta:
        """Meta class."""

        model = LocationCodeableReference
        exclude = ["created_at", "updated_at"]



"""genomic study models."""

from django.db import models

from dfhir.base.models import (
    Annotation,
    BaseReference,
    CodeableConcept,
    Identifier,
    Reference,
    TimeStampedModel,
)
from dfhir.conditions.models import ConditionObservationCodeableReference
from dfhir.encounters.models import EncounterReference
from dfhir.genomicstudies.choices import GenomicStudyStatusChoices
from dfhir.observations.models import DocumentReferenceObservationReference
from dfhir.practitioners.models import PractitionerPractitionerRoleReference
from dfhir.servicerequests.models import ServiceRequest
from dfhir.specimens.models import SpecimenReference


class GenomicStudyReference(BaseReference):
    """genomic study reference model."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="genomic_study_reference_identifier",
    )
    genomic_study = models.ForeignKey(
        "GenomicStudy",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="genomic_study_reference_genomic_study",
    )


class GenomicStudyAnalysisInput(TimeStampedModel):
    """genomic study analysis input model."""

    file = models.ForeignKey(
        "documentreferences.DocumentReferenceReference",
        on_delete=models.DO_NOTHING,
        related_name="genomic_study_analysis_input_file",
        null=True,
    )
    type = models.ForeignKey(
        CodeableConcept,
        on_delete=models.DO_NOTHING,
        related_name="genomic_study_analysis_input_type",
        null=True,
    )
    generated_by_identifier = models.ForeignKey(
        Identifier,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="genomic_study_analysis_input_generated_by_identifier",
    )
    generated_by_reference = models.ForeignKey(
        GenomicStudyReference,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="genomic_study_analysis_input_generated_by_reference",
    )


class GenomicStudyAnalysisOutput(TimeStampedModel):
    """genomic study analysis output model."""

    file = models.ForeignKey(
        "documentreferences.DocumentReferenceReference",
        on_delete=models.DO_NOTHING,
        related_name="genomic_study_analysis_output_file",
        null=True,
    )
    type = models.ForeignKey(
        CodeableConcept,
        on_delete=models.DO_NOTHING,
        related_name="genomic_study_analysis_output_type",
        null=True,
    )


class GenomicStudyAnalysisPerformerActorReference(BaseReference):
    """genomic study analysis performer actor reference model."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="genomic_study_analysis_performer_actor_reference_identifier",
    )
    practitioner = models.ForeignKey(
        "practitioners.Practitioner",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="genomic_study_analysis_performer_actor_reference_practitioner",
    )
    practitioner_role = models.ForeignKey(
        "practitionerroles.PractitionerRole",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="genomic_study_analysis_performer_actor_reference_practitioner_role",
    )
    organization = models.ForeignKey(
        "organizations.Organization",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="genomic_study_analysis_performer_actor_reference_organization",
    )
    device = models.ForeignKey(
        "devices.Device",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="genomic_study_analysis_performer_actor_reference_device",
    )


class GenomicStudyAnalysisPerformer(TimeStampedModel):
    """genomic study analysis performer model."""

    actor = models.ForeignKey(
        GenomicStudyAnalysisPerformerActorReference,
        on_delete=models.DO_NOTHING,
        related_name="genomic_study_analysis_performer_actor",
        null=True,
    )
    role = models.ForeignKey(
        CodeableConcept,
        on_delete=models.DO_NOTHING,
        related_name="genomic_study_analysis_performer_role",
        null=True,
    )


class GenomicStudyAnalysisDevice(TimeStampedModel):
    """genomic study analysis device model."""

    device = models.ForeignKey(
        "devices.DeviceReference",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="genomic_study_analysis_device_device",
    )
    function = models.ForeignKey(
        CodeableConcept,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="genomic_study_analysis_device_function",
    )


class GenomicStudyAnalysisProtocolPerformedReference(BaseReference):
    """procedure task reference model."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="procedure_task_reference_identifier",
    )
    procedure = models.ForeignKey(
        "procedures.Procedure",
        on_delete=models.DO_NOTHING,
        related_name="procedure_task_reference_procedure",
        null=True,
    )
    task = models.ForeignKey(
        "tasks.Task",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="procedure_task_reference_task",
    )


class GenomicStudyAnalysis(TimeStampedModel):
    """genomic study analysis model."""

    identifier = models.ManyToManyField(
        Identifier, blank=True, related_name="genomic_study_analysis_identifier"
    )
    method_type = models.ManyToManyField(
        CodeableConcept, blank=True, related_name="genomic_study_analysis_method_type"
    )
    change_type = models.ManyToManyField(
        CodeableConcept, blank=True, related_name="genomic_study_analysis_change_type"
    )
    genomic_build = models.ForeignKey(
        CodeableConcept,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="genomic_study_analysis_genomic_build",
    )
    # TODO: instantiates_canonical = models.ForeignKey(GenomicStudyReference)
    instantiates_uri = models.URLField(null=True)
    title = models.CharField(max_length=200, null=True)
    focus = models.ManyToManyField(
        Reference, blank=True, related_name="genomic_study_analysis_focus"
    )
    specimen = models.ManyToManyField(
        SpecimenReference, blank=True, related_name="genomic_study_analysis_specimen"
    )
    date = models.DateTimeField(null=True)
    note = models.ManyToManyField(
        Annotation, blank=True, related_name="genomic_study_analysis_note"
    )
    protocol_performed = models.ForeignKey(
        GenomicStudyAnalysisProtocolPerformedReference,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="genomic_study_analysis_protocol_performed",
    )
    regions_studied = models.ManyToManyField(
        DocumentReferenceObservationReference,
        blank=True,
        related_name="genomic_study_analysis_regions_studied",
    )
    regions_called = models.ManyToManyField(
        DocumentReferenceObservationReference,
        blank=True,
        related_name="genomic_study_analysis_regions_called",
    )
    input = models.ManyToManyField(
        GenomicStudyAnalysisInput,
        blank=True,
        related_name="genomic_study_analysis_input",
    )
    output = models.ManyToManyField(
        GenomicStudyAnalysisOutput,
        blank=True,
        related_name="genomic_study_analysis_output",
    )
    performer = models.ManyToManyField(
        GenomicStudyAnalysisPerformer,
        blank=True,
        related_name="genomic_study_analysis_performer",
    )
    device = models.ManyToManyField(
        GenomicStudyAnalysisDevice,
        blank=True,
        related_name="genomic_study_analysis_device",
    )


class GenomicStudySubjectReference(BaseReference):
    """genomic study subject reference model."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="genomic_study_subject_reference_identifier",
    )
    patient = models.ForeignKey(
        "patients.Patient",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="genomic_study_subject_reference_patient",
    )
    group = models.ForeignKey(
        "groups.Group",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="genomic_study_subject_reference_group",
    )
    substance = models.ForeignKey(
        "substances.Substance",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="genomic_study_subject_reference_substance",
    )
    biologically_derived_product = models.ForeignKey(
        "biologicallyderivedproducts.BiologicallyDerivedProduct",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="genomic_study_subject_reference_biologically_derived_product",
    )
    nutrition_product = models.ForeignKey(
        "nutritionproducts.NutritionProduct",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="genomic_study_subject_reference_nutrition_product",
    )


class GenomicStudyBasedOnReference(BaseReference):
    """genomic study based on reference model."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="genomic_study_based_on_reference_identifier",
    )
    service_request = models.ForeignKey(
        ServiceRequest,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="genomic_study_based_on_reference_service_request",
    )
    task = models.ForeignKey(
        "tasks.Task",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="genomic_study_based_on_reference_task",
    )


class GenomicStudy(TimeStampedModel):
    """genomic study model."""

    identifier = models.ManyToManyField(
        Identifier, blank=True, related_name="genomic_study_identifier"
    )
    status = models.CharField(
        max_length=200, null=True, choices=GenomicStudyStatusChoices.choices
    )
    type = models.ManyToManyField(
        CodeableConcept, blank=True, related_name="genomic_study_type"
    )
    subject = models.ForeignKey(
        GenomicStudySubjectReference,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="genomic_study_subject",
    )
    encounter = models.ForeignKey(
        EncounterReference,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="genomic_study_encounter",
    )
    start_date = models.DateTimeField(null=True)
    based_on = models.ManyToManyField(
        GenomicStudyBasedOnReference, blank=True, related_name="genomic_study_based_on"
    )
    referrer = models.ForeignKey(
        PractitionerPractitionerRoleReference,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="genomic_study_referrer",
    )
    interpreter = models.ManyToManyField(
        PractitionerPractitionerRoleReference,
        blank=True,
        related_name="genomic_study_interpreter",
    )
    reason = models.ManyToManyField(
        ConditionObservationCodeableReference,
        blank=True,
        related_name="genomic_study_reason",
    )
    # TODO: instantiates_canonical = models.ForeignKey("PlanDefinitionCanonical")
    instantiates_uri = models.URLField(null=True)
    note = models.ManyToManyField(
        Annotation, blank=True, related_name="genomic_study_note"
    )
    description = models.TextField(null=True)
    analysis = models.ManyToManyField(
        GenomicStudyAnalysis, blank=True, related_name="genomic_study_analysis"
    )



"""medication administrations views."""

from django.http import Http404
from drf_spectacular.utils import extend_schema
from rest_framework import status
from rest_framework.permissions import AllowAny
from rest_framework.response import Response
from rest_framework.views import APIView

from dfhir.medicationadministrations.models import MedicationAdministration
from dfhir.medicationadministrations.serializers import (
    MedicationAdministrationSerializer,
)


class MedicationAdministrationListView(APIView):
    """medication administration list view."""

    permission_classes = [AllowAny]

    @extend_schema(
        responses={200: MedicationAdministrationSerializer(many=True)},
    )
    def get(self, request):
        """Get."""
        queryset = MedicationAdministration.objects.all()
        serializer = MedicationAdministrationSerializer(queryset, many=True)
        return Response(serializer.data, status=status.HTTP_200_OK)

    @extend_schema(
        request=MedicationAdministrationSerializer,
        responses={201: MedicationAdministrationSerializer},
    )
    def post(self, request):
        """Post."""
        serializer = MedicationAdministrationSerializer(data=request.data)
        serializer.is_valid(raise_exception=True)
        serializer.save()
        return Response(serializer.data, status=status.HTTP_201_CREATED)


class MedicationAdministrationDetailView(APIView):
    """medication administration detail view."""

    def get_object(self, pk):
        """Get object."""
        try:
            return MedicationAdministration.objects.get(pk=pk)
        except MedicationAdministration.DoesNotExist as err:
            raise Http404 from err

    @extend_schema(responses={200: MedicationAdministrationSerializer})
    def get(self, request, pk=None):
        """Get a medication administration by PK."""
        medication_administration = self.get_object(pk)
        serializer = MedicationAdministrationSerializer(medication_administration)
        return Response(serializer.data, status=status.HTTP_200_OK)

    @extend_schema(responses={200: MedicationAdministrationSerializer})
    def patch(self, request, pk=None):
        """Patch a medication administration by PK."""
        medication_administration = self.get_object(pk)
        serializer = MedicationAdministrationSerializer(
            medication_administration, data=request.data, partial=True
        )
        serializer.is_valid(raise_exception=True)
        serializer.save()
        return Response(serializer.data, status=status.HTTP_200_OK)

    def delete(self, request, pk=None):
        """Delete a medication administration by PK."""
        medication_administration = self.get_object(pk)
        medication_administration.delete()
        return Response(status=status.HTTP_204_NO_CONTENT)



# Generated by Django 5.2 on 2025-05-05 14:45

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('activitydefinitions', '0009_initial'),
        ('base', '0005_initial'),
        ('plandefinitions', '0003_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='ActivityPlanDefinitionReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
                ('activity_definition', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='activity_plan_definition_reference_activity_definition', to='activitydefinitions.activitydefinition')),
                ('identifier', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='activity_plan_definition_reference_identifier', to='base.identifier')),
                ('plan_definition', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='activity_plan_definition_reference_plan_definition', to='plandefinitions.plandefinition')),
            ],
            options={
                'abstract': False,
            },
        ),
    ]



# Generated by Django 5.2 on 2025-05-05 14:45

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('activitydefinitions', '0010_activityplandefinitionreference'),
        ('biologicallyderivedproducts', '0002_initial'),
        ('coverages', '0002_initial'),
        ('detectedissues', '0005_initial'),
        ('devicerequests', '0004_initial'),
        ('documentreferences', '0005_initial'),
        ('nutritionorders', '0004_nutritionorder_allergy_intolerance'),
        ('plandefinitions', '0003_initial'),
        ('servicerequests', '0001_initial'),
        ('specimens', '0001_initial'),
        ('substances', '0001_initial'),
        ('supplyrequests', '0001_initial'),
    ]

    operations = [
        migrations.RenameField(
            model_name='servicerequestbasedonreference',
            old_name='medicatoin_request',
            new_name='medication_request',
        ),
        migrations.AddField(
            model_name='orderdetailparameterfocusreference',
            name='biological_derived_product',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='order_detail_parameter_focus_reference_biological_derived_product', to='biologicallyderivedproducts.biologicallyderivedproduct'),
        ),
        migrations.AddField(
            model_name='orderdetailparameterfocusreference',
            name='device_request',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='order_detail_parameter_focus_device_request', to='devicerequests.devicerequest'),
        ),
        migrations.AddField(
            model_name='orderdetailparameterfocusreference',
            name='substance',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='order_detail_parameter_focus_substance', to='substances.substance'),
        ),
        migrations.AddField(
            model_name='orderdetailparameterfocusreference',
            name='supply_request',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='order_detail_parameter_focus_supply_request', to='supplyrequests.supplyrequest'),
        ),
        migrations.AddField(
            model_name='servicerequest',
            name='code',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='service_request_code', to='activitydefinitions.activityplandefinitionreference'),
        ),
        migrations.AddField(
            model_name='servicerequest',
            name='insurance',
            field=models.ManyToManyField(blank=True, related_name='service_request_insurance', to='coverages.coverageclaimresponsereference'),
        ),
        migrations.AddField(
            model_name='servicerequest',
            name='specimen',
            field=models.ManyToManyField(blank=True, related_name='service_request_specimen', to='specimens.specimenreference'),
        ),
        migrations.AddField(
            model_name='servicerequestbasedonreference',
            name='nutrition_order',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='service_request_based_on_nutrition_order', to='nutritionorders.nutritionorder'),
        ),
        migrations.AddField(
            model_name='servicerequestpatientinstruction',
            name='instruction_reference',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='patient_instruction_document_reference', to='documentreferences.documentreference'),
        ),
        migrations.AddField(
            model_name='servicerequestplandefinitionreference',
            name='plan_definition',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='service_request_plan_definition_plan_definition', to='plandefinitions.plandefinition'),
        ),
        migrations.AddField(
            model_name='servicerequestreasonreference',
            name='detected_issue',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='service_request_reason_detected_issue', to='detectedissues.detectedissue'),
        ),
        migrations.AddField(
            model_name='servicerequestreasonreference',
            name='document_reference',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='service_request_reason_document_reference', to='documentreferences.documentreference'),
        ),
    ]



# Generated by Django 5.2 on 2025-04-23 12:46

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('base', '0005_initial'),
        ('patients', '0002_initial'),
        ('personalrelationships', '0001_initial'),
        ('persons', '0001_initial'),
        ('practitioners', '0001_initial'),
        ('relatedpersons', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='personalrelationshipasserterreference',
            name='practitioner',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='personal_relationship_asserter_reference_practitioner', to='practitioners.practitioner'),
        ),
        migrations.AddField(
            model_name='personalrelationshipasserterreference',
            name='related_person',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='personal_relationship_asserter_reference_related_person', to='relatedpersons.relatedperson'),
        ),
        migrations.CreateModel(
            name='PersonalRelationshipTargetReference',
            fields=[
                ('personalrelationshipsourcereference_ptr', models.OneToOneField(auto_created=True, on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, serialize=False, to='personalrelationships.personalrelationshipsourcereference')),
            ],
            options={
                'abstract': False,
            },
            bases=('personalrelationships.personalrelationshipsourcereference',),
        ),
        migrations.AddField(
            model_name='personalrelationshipsourcereference',
            name='identifier',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='personal_relationship_source_reference_identifier', to='base.identifier'),
        ),
        migrations.AddField(
            model_name='personalrelationshipsourcereference',
            name='patient',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='personal_relationship_source_reference_patient', to='patients.patient'),
        ),
        migrations.AddField(
            model_name='personalrelationshipsourcereference',
            name='person',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='personal_relationship_source_reference_person', to='persons.person'),
        ),
        migrations.AddField(
            model_name='personalrelationshipsourcereference',
            name='related_person',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='personal_relationship_source_reference_related_person', to='relatedpersons.relatedperson'),
        ),
        migrations.AddField(
            model_name='personalrelationship',
            name='source',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='personal_relationships_source', to='personalrelationships.personalrelationshipsourcereference'),
        ),
        migrations.AddField(
            model_name='personalrelationship',
            name='target',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='personal_relationships_target', to='personalrelationships.personalrelationshiptargetreference'),
        ),
    ]



# Generated by Django 5.2 on 2025-04-23 12:46

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('base', '0005_initial'),
        ('medicationstatements', '0002_initial'),
        ('practitionerroles', '0002_initial'),
        ('practitioners', '0001_initial'),
        ('relatedpersons', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='medicationstatementinformationsourcereference',
            name='practitioner',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='medication_statement_information_source_reference_practitioner', to='practitioners.practitioner'),
        ),
        migrations.AddField(
            model_name='medicationstatementinformationsourcereference',
            name='practitioner_role',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='medication_statement_information_source_reference_practitioner_role', to='practitionerroles.practitionerrole'),
        ),
        migrations.AddField(
            model_name='medicationstatementinformationsourcereference',
            name='related_person',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='medication_statement_information_source_reference_related_person', to='relatedpersons.relatedperson'),
        ),
        migrations.AddField(
            model_name='medicationstatement',
            name='information_source',
            field=models.ManyToManyField(blank=True, related_name='medication_statement_information_source', to='medicationstatements.medicationstatementinformationsourcereference'),
        ),
        migrations.AddField(
            model_name='medicationstatementpartofreference',
            name='identifier',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='medication_statement_part_of_reference_identifier', to='base.identifier'),
        ),
        migrations.AddField(
            model_name='medicationstatementpartofreference',
            name='medication_statement',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='medication_statement_part_of_reference_medication_statement', to='medicationstatements.medicationstatement'),
        ),
    ]



"""biologically derived product dispense views."""

from django.http import Http404
from drf_spectacular.utils import extend_schema
from rest_framework import status
from rest_framework.permissions import AllowAny
from rest_framework.response import Response
from rest_framework.views import APIView

from dfhir.biologicallyderivedproductdispenses.models import (
    BiologicallyDerivedProductDispense,
)
from dfhir.biologicallyderivedproductdispenses.serializers import (
    BiologicallyDerivedProductDispenseSerializer,
)


class BiologicallyDerivedProductDispenseListView(APIView):
    """biologically derived product dispense list view."""

    permission_classes = [AllowAny]

    @extend_schema(
        responses={200: BiologicallyDerivedProductDispenseSerializer(many=True)}
    )
    def get(self, request):
        """Get a list of biologically derived product dispenses."""
        biologically_derived_product_dispenses = (
            BiologicallyDerivedProductDispense.objects.all()
        )
        serializer = BiologicallyDerivedProductDispenseSerializer(
            biologically_derived_product_dispenses, many=True
        )
        return Response(serializer.data, status=status.HTTP_200_OK)

    @extend_schema(
        request=BiologicallyDerivedProductDispenseSerializer,
        responses={201: BiologicallyDerivedProductDispenseSerializer},
    )
    def post(self, request):
        """Create a biologically derived product dispense."""
        serializer = BiologicallyDerivedProductDispenseSerializer(data=request.data)
        serializer.is_valid(raise_exception=True)
        serializer.save()
        return Response(serializer.data, status=status.HTTP_201_CREATED)


class BiologicallyDerivedProductDispenseDetailView(APIView):
    """biologically derived product dispense detail view."""

    def get_object(self, pk):
        """Get biologically derived product dispense object."""
        try:
            return BiologicallyDerivedProductDispense.objects.get(pk=pk)
        except BiologicallyDerivedProductDispense.DoesNotExist as err:
            raise Http404 from err

    @extend_schema(responses={200: BiologicallyDerivedProductDispenseSerializer})
    def get(self, request, pk=None):
        """Get a biologically derived product dispense."""
        biologically_derived_product_dispense = self.get_object(pk)
        serializer = BiologicallyDerivedProductDispenseSerializer(
            biologically_derived_product_dispense
        )
        return Response(serializer.data, status=status.HTTP_200_OK)

    @extend_schema(
        request=BiologicallyDerivedProductDispenseSerializer,
        responses={200: BiologicallyDerivedProductDispenseSerializer},
    )
    def patch(self, request, pk=None):
        """Update a biologically derived product dispense."""
        biologically_derived_product_dispense = self.get_object(pk)
        serializer = BiologicallyDerivedProductDispenseSerializer(
            biologically_derived_product_dispense, data=request.data, partial=True
        )
        serializer.is_valid(raise_exception=True)
        serializer.save()
        return Response(serializer.data, status=status.HTTP_200_OK)

    @extend_schema(responses={204: None})
    def delete(self, request, pk=None):
        """Delete a biologically derived product dispense."""
        biologically_derived_product_dispense = self.get_object(pk)
        biologically_derived_product_dispense.delete()
        return Response(status=status.HTTP_204_NO_CONTENT)



# Generated by Django 5.2 on 2025-04-23 12:46

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('base', '0005_initial'),
        ('biologicallyderivedproductdispenses', '0001_initial'),
        ('locations', '0001_initial'),
        ('patients', '0002_initial'),
        ('practitioners', '0001_initial'),
        ('servicerequests', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='biologicallyderivedproductdispense',
            name='based_on',
            field=models.ManyToManyField(blank=True, related_name='biologicallyderivedproductdispense_based_on', to='servicerequests.servicerequestreference'),
        ),
        migrations.AddField(
            model_name='biologicallyderivedproductdispense',
            name='destination',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='biologicallyderivedproductdispense_destination', to='locations.locationreference'),
        ),
        migrations.AddField(
            model_name='biologicallyderivedproductdispense',
            name='identifier',
            field=models.ManyToManyField(blank=True, related_name='biologicallyderivedproductdispense_identifier', to='base.identifier'),
        ),
        migrations.AddField(
            model_name='biologicallyderivedproductdispense',
            name='location',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='biologicallyderivedproductdispense_location', to='locations.locationreference'),
        ),
        migrations.AddField(
            model_name='biologicallyderivedproductdispense',
            name='match_status',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='biologicallyderivedproductdispense_match_status', to='base.codeableconcept'),
        ),
        migrations.AddField(
            model_name='biologicallyderivedproductdispense',
            name='note',
            field=models.ManyToManyField(blank=True, related_name='biologicallyderivedproductdispense_note', to='base.annotation'),
        ),
        migrations.AddField(
            model_name='biologicallyderivedproductdispense',
            name='original_relationship_type',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='biologicallyderivedproductdispense_original_relationship_type', to='base.codeableconcept'),
        ),
        migrations.AddField(
            model_name='biologicallyderivedproductdispense',
            name='patient',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='biologicallyderivedproductdispense_patient', to='patients.patientreference'),
        ),
        migrations.AddField(
            model_name='biologicallyderivedproductdispense',
            name='quantity',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='biologicallyderivedproductdispense_quantity', to='base.simplequantity'),
        ),
        migrations.AddField(
            model_name='biologicallyderivedproductdispenseperformer',
            name='actor',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='biologicallyderivedproductdispense_performer_actor', to='practitioners.practitionerreference'),
        ),
        migrations.AddField(
            model_name='biologicallyderivedproductdispenseperformer',
            name='function',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='biologicallyderivedproductdispense_performer_function', to='base.codeableconcept'),
        ),
        migrations.AddField(
            model_name='biologicallyderivedproductdispense',
            name='performer',
            field=models.ManyToManyField(blank=True, related_name='biologicallyderivedproductdispense_performer', to='biologicallyderivedproductdispenses.biologicallyderivedproductdispenseperformer'),
        ),
        migrations.AddField(
            model_name='biologicallyderivedproductdispensereference',
            name='biologically_derived_product_dispense',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='biologicallyderivedproductdispense_reference_biologicallyderivedproductdispense', to='biologicallyderivedproductdispenses.biologicallyderivedproductdispense'),
        ),
        migrations.AddField(
            model_name='biologicallyderivedproductdispensereference',
            name='identifier',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='biologicallyderivedproductdispense_reference_identifier', to='base.identifier'),
        ),
        migrations.AddField(
            model_name='biologicallyderivedproductdispense',
            name='part_of',
            field=models.ManyToManyField(blank=True, related_name='biologicallyderivedproductdispense_part_of', to='biologicallyderivedproductdispenses.biologicallyderivedproductdispensereference'),
        ),
        migrations.AddField(
            model_name='biologicallyderivedproductdispense',
            name='product',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='biologicallyderivedproductdispense_product', to='biologicallyderivedproductdispenses.biologicallyderivedproductdispensereference'),
        ),
    ]



"""task models."""

from django.db import models

from dfhir.base.models import (
    BaseReference,
    CodeableConcept,
    CodeableReference,
    Identifier,
    TimeStampedModel,
)
from dfhir.coverages.models import CoverageClaimResponseReference
from dfhir.provenances.models import ProvenanceReference
from dfhir.tasks.choices import (
    TaskIntentChoices,
    TaskPriorityChoices,
    TaskStatusChoices,
)


class TaskReference(BaseReference):
    """task reference model."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="task_reference_identifier",
    )
    task = models.ForeignKey(
        "Task",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="task_reference_task",
    )


class TaskRequesterReference(BaseReference):
    """task requester reference."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="task_requester_reference_identifier",
    )
    device = models.ForeignKey(
        "devices.Device",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="task_requester_reference_device",
    )
    organization = models.ForeignKey(
        "organizations.Organization",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="task_requester_reference_organization",
    )
    patient = models.ForeignKey(
        "patients.Patient",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="task_requester_reference_patient",
    )
    practitioner = models.ForeignKey(
        "practitioners.Practitioner",
        null=True,
        on_delete=models.DO_NOTHING,
        related_name="task_requester_reference_practitioner",
    )
    practitioner_role = models.ForeignKey(
        "practitionerroles.PractitionerRole",
        null=True,
        on_delete=models.DO_NOTHING,
        related_name="task_requester_reference_practitioner_role",
    )
    related_person = models.ForeignKey(
        "relatedpersons.RelatedPerson",
        null=True,
        on_delete=models.DO_NOTHING,
        related_name="task_requester_reference_related_person",
    )


class TaskRequestedPerformerReference(BaseReference):
    """task requested performer reference model."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="task_requested_performer_reference_identifier",
    )
    practitioner = models.ForeignKey(
        "practitioners.Practitioner",
        null=True,
        on_delete=models.DO_NOTHING,
        related_name="task_requested_performer_reference_practitioner",
    )
    practitioner_role = models.ForeignKey(
        "practitionerroles.PractitionerRole",
        null=True,
        on_delete=models.DO_NOTHING,
        related_name="task_requested_performer_reference_practitioner_role",
    )
    organization = models.ForeignKey(
        "organizations.Organization",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="task_requested_performer_reference_organization",
    )
    care_team = models.ForeignKey(
        "careteams.CareTeam",
        on_delete=models.DO_NOTHING,
        related_name="task_requested_practitioner_care_team",
        null=True,
    )
    healthcare_service = models.ForeignKey(
        "healthcareservices.HealthcareService",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="task_requested_practitioner_healthcare_service",
    )
    patient = models.ForeignKey(
        "patients.Patient",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="task_requested_performer_reference_patient",
    )
    device = models.ForeignKey(
        "devices.Device",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="task_requested_performer_reference_device",
    )
    related_person = models.ForeignKey(
        "relatedpersons.RelatedPerson",
        null=True,
        on_delete=models.DO_NOTHING,
        related_name="task_requested_performer_reference_related_person",
    )


class TaskRequestedPerformerCodeableReference(TimeStampedModel):
    """task requested performer codeable reference."""

    concept = models.ForeignKey(
        CodeableConcept,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="task_requested_performer_codeable_reference_concept",
    )
    reference = models.ForeignKey(
        TaskRequestedPerformerReference,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="task_requested_performer_codeable_reference_reference",
    )


class TaskOwnerReference(BaseReference):
    """task owner reference model."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="task_owner_reference_identifier",
    )
    practitioner = models.ForeignKey(
        "practitioners.Practitioner",
        null=True,
        on_delete=models.DO_NOTHING,
        related_name="task_owner_reference_practitioner",
    )
    practitioner_role = models.ForeignKey(
        "practitionerroles.PractitionerRole",
        null=True,
        on_delete=models.DO_NOTHING,
        related_name="task_owner_reference_practitioner_role",
    )
    organization = models.ForeignKey(
        "organizations.Organization",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="task_owner_reference_organization",
    )
    care_team = models.ForeignKey(
        "careteams.CareTeam",
        on_delete=models.DO_NOTHING,
        related_name="task_owner_practitioner_care_team",
        null=True,
    )
    patient = models.ForeignKey(
        "patients.Patient",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="task_owner_reference_patient",
    )
    related_person = models.ForeignKey(
        "relatedpersons.RelatedPerson",
        null=True,
        on_delete=models.DO_NOTHING,
        related_name="task_owner_reference_related_person",
    )


class TaskPerformerActorReference(BaseReference):
    """task performer actor reference model."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.DO_NOTHING,
        related_name="task_performer_actor_reference_identifier",
        null=True,
    )
    practitioner = models.ForeignKey(
        "practitioners.Practitioner",
        null=True,
        on_delete=models.DO_NOTHING,
        related_name="task_performer_actor_reference_practitioner",
    )
    device = models.ForeignKey(
        "devices.Device",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="task_performer_actor_reference_device",
    )

    organization = models.ForeignKey(
        "organizations.Organization",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="task_performer_actor_reference_organization",
    )
    practitioner_role = models.ForeignKey(
        "practitionerroles.PractitionerRole",
        null=True,
        on_delete=models.DO_NOTHING,
        related_name="task_performer_actor_reference_practitioner_role",
    )
    care_team = models.ForeignKey(
        "careteams.CareTeam",
        on_delete=models.DO_NOTHING,
        related_name="task_performer_actor_reference_practitioner_care_team",
        null=True,
    )
    patient = models.ForeignKey(
        "patients.Patient",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="task_performer_actor_reference_patient",
    )

    related_person = models.ForeignKey(
        "relatedpersons.RelatedPerson",
        null=True,
        on_delete=models.DO_NOTHING,
        related_name="task_performer_actor_reference_related_person",
    )


class TaskRestrictionRecipientReference(BaseReference):
    """task restriction recipient reference model."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.DO_NOTHING,
        related_name="task_restriction_recipient_reference",
        null=True,
    )
    practitioner = models.ForeignKey(
        "practitioners.Practitioner",
        null=True,
        on_delete=models.DO_NOTHING,
        related_name="task_restriction_recipient_reference_practitioner",
    )
    practitioner_role = models.ForeignKey(
        "practitionerroles.PractitionerRole",
        null=True,
        on_delete=models.DO_NOTHING,
        related_name="task_restriction_recipient_reference_practitioner_role",
    )
    organization = models.ForeignKey(
        "organizations.Organization",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="task_restriction_recipient_reference_organization",
    )
    group = models.ForeignKey(
        "groups.Group",
        on_delete=models.DO_NOTHING,
        related_name="task_requested_practitioner_care_team",
        null=True,
    )
    patient = models.ForeignKey(
        "patients.Patient",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="task_restriction_recipient_reference_patient",
    )
    related_person = models.ForeignKey(
        "relatedpersons.RelatedPerson",
        null=True,
        on_delete=models.DO_NOTHING,
        related_name="task_restriction_recipient_reference_related_person",
    )


class TaskPerformer(TimeStampedModel):
    """task performer model."""

    function = models.ForeignKey(
        "base.CodeableConcept",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="task_performer_model",
    )
    actor = models.ForeignKey(
        TaskPerformerActorReference,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="task_performer_actor",
    )


class TaskRestriction(TimeStampedModel):
    """task restriction model."""

    repetition = models.PositiveIntegerField(null=True)
    period = models.ForeignKey(
        "base.Period",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="task_restriction_period",
    )
    recipient = models.ManyToManyField(
        TaskRestrictionRecipientReference,
        blank=True,
        related_name="ask_restrictions_recipient",
    )


class TaskInput(TimeStampedModel):
    """task input model."""

    type = models.ForeignKey(
        CodeableConcept,
        null=False,
        on_delete=models.DO_NOTHING,
        related_name="task_input_type",
    )
    value = models.CharField(null=True, max_length=255)


class TaskOutput(TimeStampedModel):
    """task output model."""

    type = models.ForeignKey(
        CodeableConcept,
        null=False,
        on_delete=models.DO_NOTHING,
        related_name="task_output_type",
    )
    value = models.CharField(null=True, max_length=255)


class Task(TimeStampedModel):
    """task model."""

    identifier = models.ManyToManyField(
        Identifier, blank=False, related_name="task_identifier"
    )
    # TODO: instantiate_canonical = models.ForeignKey(
    #     "ActivityDefinitionCAnonical",
    #     related_name="task_instantiate_canonical",
    #     null=True,
    #     on_delete=models.DO_NOTHING,
    # )
    instantiate_uri = models.URLField(null=True)
    based_on = models.ManyToManyField(
        "base.Reference", blank=True, related_name="task_based_on"
    )
    group_identifier = models.ForeignKey(
        Identifier,
        on_delete=models.DO_NOTHING,
        related_name="task_group_identifier",
        null=True,
    )
    part_of = models.ManyToManyField(
        TaskReference, blank=True, related_name="task_part_of"
    )
    status = models.CharField(
        max_length=255, null=True, choices=TaskStatusChoices.choices
    )
    status_reason = models.ForeignKey(
        CodeableReference,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="task_status_reason",
    )
    business_status = models.ForeignKey(
        CodeableConcept,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="task_business_status",
    )
    intent = models.CharField(
        max_length=255, null=True, choices=TaskIntentChoices.choices
    )
    priority = models.CharField(
        max_length=255, null=True, choices=TaskPriorityChoices.choices
    )
    do_not_perform = models.BooleanField(default=False)
    code = models.ForeignKey(
        CodeableConcept,
        on_delete=models.DO_NOTHING,
        related_name="task_code",
        null=True,
    )
    description = models.TextField(null=True)
    focus = models.ForeignKey(
        "base.Reference",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="task_focus",
    )
    for_value = models.ForeignKey(
        "base.Reference",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="task_for",
    )
    encounter = models.ForeignKey(
        "encounters.EncounterReference",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="task_encounter",
    )
    requested_period = models.ForeignKey(
        "base.Period",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="task_requested_period",
    )
    execution_period = models.ForeignKey(
        "base.Period",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="task_execution_period",
    )
    authored_on = models.DateTimeField(null=True)
    last_modified = models.DateTimeField(null=True)
    requester = models.ForeignKey(
        TaskRequesterReference,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="task_requester",
    )
    requested_performer = models.ManyToManyField(
        TaskRequestedPerformerCodeableReference,
        blank=True,
        related_name="task_requested_performer",
    )
    owner = models.ForeignKey(
        TaskOwnerReference,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="task_owner",
    )
    performer = models.ManyToManyField(
        TaskPerformer, related_name="task_performer", blank=True
    )
    location = models.ForeignKey(
        "locations.LocationReference",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="task_location",
    )
    reason = models.ManyToManyField(
        CodeableReference, blank=True, related_name="task_reason"
    )
    insurance = models.ManyToManyField(
        CoverageClaimResponseReference, blank=True, related_name="task_insurance"
    )
    note = models.ManyToManyField(
        "base.Annotation", blank=True, related_name="task_note"
    )
    relevant_history = models.ManyToManyField(
        ProvenanceReference, blank=True, related_name="task_relevant_history"
    )
    restriction = models.ForeignKey(
        TaskRestriction,
        null=True,
        on_delete=models.DO_NOTHING,
        related_name="task_restriction",
    )
    input = models.ManyToManyField(TaskInput, blank=True, related_name="task_input")
    output = models.ManyToManyField(TaskOutput, blank=True, related_name="task_output")



"""Appointments views."""

from django.http import Http404
from drf_spectacular.utils import extend_schema
from rest_framework import status
from rest_framework.permissions import AllowAny
from rest_framework.response import Response
from rest_framework.views import APIView

from .models import Appointment
from .serializers import AppointmentSerializer


class AppointmentListView(APIView):
    """Appointment list view."""

    permission_classes = [AllowAny]

    @extend_schema(responses={200: AppointmentSerializer(many=True)})
    def get(self, request):
        """Get all appointments."""
        appointments = Appointment.objects.all()
        serializer = AppointmentSerializer(appointments, many=True)
        return Response(serializer.data, status=status.HTTP_200_OK)

    @extend_schema(
        request=AppointmentSerializer, responses={201: AppointmentSerializer}
    )
    def post(self, request):
        """Create an appointment."""
        serializer = AppointmentSerializer(data=request.data)
        serializer.is_valid(raise_exception=True)
        appointment = serializer.save()
        serializer = AppointmentSerializer(appointment)
        return Response(serializer.data, status=status.HTTP_201_CREATED)


class AppointmentDetailView(APIView):
    """Appointment detail view."""

    permission_classes = [AllowAny]

    def get_object(self, pk):
        """Get an appointment object."""
        try:
            return Appointment.objects.get(pk=pk)
        except Appointment.DoesNotExist as err:
            raise Http404 from err

    @extend_schema(responses={200, AppointmentSerializer})
    def get(self, request, pk=None):
        """Get an appointment."""
        appointment = self.get_object(pk)
        serializer = AppointmentSerializer(appointment)
        return Response(serializer.data, status=status.HTTP_200_OK)

    @extend_schema(responses={200: AppointmentSerializer})
    def patch(self, request, pk=None):
        """Update an appointment."""
        appointment = self.get_object(pk)
        serializer = AppointmentSerializer(appointment, data=request.data, partial=True)
        serializer.is_valid(raise_exception=True)
        appointment = serializer.save()
        serializer = AppointmentSerializer(appointment)
        return Response(serializer.data, status=status.HTTP_200_OK)

    def delete(self, request, pk=None):
        """Delete an appointment."""
        appointment = self.get_object(pk)
        appointment.delete()
        return Response(status=status.HTTP_204_NO_CONTENT)



"""payment reconciliations model choices."""

from django.db import models
from django.utils.translation import gettext_lazy as _


class PaymentReconciliationStatusChoices(models.TextChoices):
    """Payment Reconciliation status choices."""

    ACTIVE = "active", _("Active")
    CANCELLED = "cancelled", _("Cancelled")
    DRAFT = "draft", _("Draft")
    ENTERED_IN_ERROR = "entered-in-error", _("Entered in Error")


class PaymentReconciliationOutcomeChoices(models.TextChoices):
    """Payment Reconciliation outcome choices."""

    QUEUED = "queued", _("Queued")
    COMPLETE = "complete", _("Complete")
    ERROR = "error", _("Error")
    PARTIAL = "partial", _("Partial")


class PaymentReconciliationProcessNoteChoices(models.TextChoices):
    """Payment Reconciliation Process Note choices."""

    DISPLAY = "display", _("Display")
    PRINT = "print", _("Print")
    PRINTOPER = "printoper", _("Print Oper")



# Generated by Django 5.2 on 2025-04-23 12:46

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Coverage',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('status', models.CharField(choices=[('active', 'Active'), ('cancelled', 'Cancelled'), ('draft', 'Draft'), ('entered-in-error', 'Entered in Error')], max_length=255)),
                ('kind', models.CharField(choices=[('insurance', 'Insurance'), ('self-pay', 'Self Pay'), ('other', 'Other')], max_length=255)),
                ('dependent', models.CharField(max_length=255, null=True)),
                ('order', models.IntegerField(null=True)),
                ('network', models.CharField(max_length=255, null=True)),
                ('subrogation', models.BooleanField(default=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='CoverageClaimResponseReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='CoverageClass',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('name', models.CharField(max_length=255, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='CoverageCostToBeneficiary',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='CoverageCostToBeneficiaryException',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='CoveragePaymentBy',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('responsibility', models.CharField(max_length=255, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='CoveragePaymentByPartyReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='CoveragePolicyHolderReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='CoverageReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
    ]



"""Biolocically Derived Products serializers."""

from dfhir.base.serializers import (
    AttachmentSerializer,
    BaseReferenceModelSerializer,
    BaseWritableNestedModelSerializer,
    CodeableConceptSerializer,
    CodingSerializer,
    IdentifierSerializer,
    OrganizationReferenceSerializer,
    PeriodSerializer,
    QuantitySerializer,
    RangeSerializer,
    RatioSerializer,
)
from dfhir.patients.serializers import PatientOrganizationReferenceSerializer
from dfhir.practitioners.serializers import (
    PractitionerPractitionerRoleReferenceSerializer,
)
from dfhir.procedures.serializers import ProcedureSerializer
from dfhir.servicerequests.serializers import ServiceRequestReferenceSerializer

from .models import (
    BiologicallyDerivedProduct,
    BiologicallyDerivedProductCollection,
    BiologicallyDerivedProductProperty,
    BiologicallyDerivedProductReference,
)


class BiologicallyDerivedProductReferenceSerializer(BaseReferenceModelSerializer):
    """Biologically Derived Product Reference serializer."""

    identifier = IdentifierSerializer(required=False)

    class Meta:
        """Meta class."""

        model = BiologicallyDerivedProductReference
        exclude = ["created_at", "updated_at"]


class BiologicallyDerivedProductCollectionSerializer(BaseWritableNestedModelSerializer):
    """Biologically Derived Product Collection serializer."""

    collector = PractitionerPractitionerRoleReferenceSerializer(required=False)
    source = PatientOrganizationReferenceSerializer(required=False)
    collected_period = PeriodSerializer(required=False)
    procedure = ProcedureSerializer(required=False)

    class Meta:
        """Meta class."""

        model = BiologicallyDerivedProductCollection
        exclude = ["created_at", "updated_at"]


class BiologicallyDerivedProductPropertySerializer(BaseWritableNestedModelSerializer):
    """Biologically Derived Product Property serializer."""

    type = CodeableConceptSerializer(required=False)
    value_codeable_concept = CodeableConceptSerializer(required=False)
    value_period = PeriodSerializer(required=False)
    value_quantity = QuantitySerializer(required=False)
    value_range = RangeSerializer(required=False)
    value_ratio = RatioSerializer(required=False)
    value_attachment = AttachmentSerializer(required=False)

    class Meta:
        """Meta class."""

        model = BiologicallyDerivedProductProperty
        exclude = ["created_at", "updated_at"]


class BiologicallyDerivedProductSerializer(BaseWritableNestedModelSerializer):
    """Biologically Derived Product serializer."""

    product_category = CodeableConceptSerializer(required=False, many=True)
    product_code = CodeableConceptSerializer(required=False)
    parent = BiologicallyDerivedProductReferenceSerializer(required=False, many=True)
    request = ServiceRequestReferenceSerializer(required=False, many=True)
    identifier = IdentifierSerializer(many=True, required=False)
    biological_source_event = IdentifierSerializer(required=False)
    processing_facility = OrganizationReferenceSerializer(required=False, many=True)
    product_status = CodingSerializer(required=False)
    collection = BiologicallyDerivedProductCollectionSerializer(required=False)
    storage_temp_requirements = RangeSerializer(required=False)
    property = BiologicallyDerivedProductPropertySerializer(required=False, many=True)

    class Meta:
        """Meta class."""

        model = BiologicallyDerivedProduct
        exclude = ["created_at", "updated_at"]



# Generated by Django 5.2 on 2025-04-23 12:46

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('accounts', '0003_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='EpisodeOfCareDiagnosis',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='EpisodeOfCareReason',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='EpisodeOfCareReasonValueCodeableReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='EpisodeOfCareReasonValueReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='EpisodeOfCareReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='EpisodeOfCareStatusHistory',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('status', models.CharField(choices=[('planned', 'Planned'), ('waitlist', 'Waitlist'), ('active', 'Active'), ('onhold', 'On Hold'), ('finished', 'Finished'), ('cancelled', 'Cancelled'), ('entered-in-error', 'Entered in Error')], max_length=255, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='EpisodeOfCare',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('status', models.CharField(choices=[('planned', 'Planned'), ('waitlist', 'Waitlist'), ('active', 'Active'), ('onhold', 'On Hold'), ('finished', 'Finished'), ('cancelled', 'Cancelled'), ('entered-in-error', 'Entered in Error')], max_length=255, null=True)),
                ('account', models.ManyToManyField(blank=True, related_name='episode_of_care_account', to='accounts.accountreference')),
            ],
            options={
                'abstract': False,
            },
        ),
    ]



"""Device definition models."""

from django.db import models

from dfhir.base.models import (
    Attachment,
    BaseReference,
    CodeableConcept,
    Coding,
    ContactDetail,
    Identifier,
    OrganizationReference,
    Period,
    ProductShelfLife,
    Quantity,
    Range,
    RelatedArtifact,
    TimeStampedModel,
    UsageContext,
)

from . import choices


class DeviceDefinitionMarketDistribution(TimeStampedModel):
    """DeviceDefinitionMarketDistribution model."""

    market_period = models.ForeignKey(Period, on_delete=models.CASCADE, null=True)
    sub_jurisdiction = models.CharField(max_length=255, null=True)


class DeviceDefinitionUdiDeviceIdentifier(TimeStampedModel):
    """DeviceDefinitionUdiDeviceIdentifier model."""

    device_identifier = models.CharField(max_length=255)
    issuer = models.CharField(max_length=255)
    jurisdiction = models.CharField(max_length=255)
    market_distribution = models.ManyToManyField(
        DeviceDefinitionMarketDistribution, blank=True
    )


class DeviceDefinitionRegulatoryIdentifier(TimeStampedModel):
    """DeviceDefinitionRegulatoryIdentifier model."""

    type = models.CharField(
        max_length=255, choices=choices.DeviceDefinitionRegulatoryIdentifierType.choices
    )
    device_identifier = models.CharField(max_length=255)
    issuer = models.CharField(max_length=255)
    jurisdiction = models.CharField(max_length=255)


class DeviceDefinitionDeviceName(TimeStampedModel):
    """DeviceDefinitionDeviceName model."""

    name = models.CharField(max_length=255)
    type = models.ForeignKey(
        CodeableConcept,
        related_name="device_definition_device_name_type",
        on_delete=models.CASCADE,
        null=True,
    )


class DeviceDefinitionClassification(TimeStampedModel):
    """DeviceDefinitionClassification model."""

    type = models.ForeignKey(
        CodeableConcept,
        related_name="device_definition_classification_type",
        on_delete=models.CASCADE,
        null=True,
    )
    justification = models.ManyToManyField(
        RelatedArtifact,
        related_name="device_definition_classification_jurisdication",
        blank=True,
    )


class DeviceDefinitionConformsTo(TimeStampedModel):
    """DeviceDefinitionConformsTo model."""

    category = models.ForeignKey(
        CodeableConcept,
        related_name="device_definition_conforms_to_category",
        on_delete=models.CASCADE,
        null=True,
    )
    specification = models.ForeignKey(
        CodeableConcept,
        related_name="device_definition_conforms_to_specification",
        on_delete=models.CASCADE,
        null=True,
    )
    version = models.CharField(max_length=255, null=True)
    source = models.ManyToManyField(
        RelatedArtifact, related_name="device_definition_conforms_to_source", blank=True
    )


class DeviceDefinitionHasPart(TimeStampedModel):
    """DeviceDefinitionHasPart model."""

    reference = models.ForeignKey(
        "DeviceDefinitionReference",
        related_name="device_definition_has_part_reference",
        on_delete=models.CASCADE,
        null=True,
    )
    count = models.IntegerField(null=True)


class DeviceDefinitionReference(BaseReference):
    """Device definition reference."""

    identifier = models.ForeignKey(
        Identifier,
        related_name="device_definition_reference_identifier",
        on_delete=models.CASCADE,
        null=True,
    )
    device_definition = models.ForeignKey(
        "DeviceDefinition",
        related_name="device_definition_reference_device_definition",
        on_delete=models.CASCADE,
        null=True,
    )


class DeviceDefinitionCodeableReference(TimeStampedModel):
    """DeviceDefinitionCodeableReference model."""

    reference = models.ForeignKey(
        DeviceDefinitionReference,
        related_name="device_definition_codeable_reference_reference",
        on_delete=models.CASCADE,
        null=True,
    )
    concept = models.ForeignKey(
        CodeableConcept,
        related_name="device_definition_codeable_reference_concept",
        on_delete=models.CASCADE,
        null=True,
    )


class DeviceDefinitionPackagingDistributor(TimeStampedModel):
    """Device definition packaging distributor model."""

    name = models.CharField(max_length=255)
    organization_reference = models.ForeignKey(
        OrganizationReference,
        related_name="device_definition_packaging_distributor_organization_reference",
        on_delete=models.CASCADE,
        null=True,
    )


class DeviceDefinitionPackaging(TimeStampedModel):
    """Device definition packaging model."""

    identifier = models.ForeignKey(
        Identifier,
        related_name="device_definition_packaging_identifier",
        on_delete=models.CASCADE,
        null=True,
    )
    type = models.ForeignKey(
        CodeableConcept,
        related_name="device_definition_packaging_type",
        on_delete=models.CASCADE,
        null=True,
    )
    count = models.IntegerField(null=True)
    distributor = models.ManyToManyField(
        DeviceDefinitionPackagingDistributor,
        related_name="device_definition_packaging_distributor",
        blank=True,
    )
    udi_device_identifier = models.ManyToManyField(
        DeviceDefinitionUdiDeviceIdentifier,
        related_name="device_definition_packaging_udihistory",
        blank=True,
    )
    packaging = models.ForeignKey(
        "self",
        related_name="device_definition_packaging_packaging",
        on_delete=models.CASCADE,
        null=True,
    )


class DeviceDefinitionDeviceVersion(TimeStampedModel):
    """DeviceVersion model."""

    type = models.ForeignKey(
        CodeableConcept,
        related_name="device_definition_device_version_type",
        on_delete=models.CASCADE,
        null=True,
    )
    component = models.ForeignKey(
        Identifier,
        related_name="device_definition_device_version_component",
        on_delete=models.CASCADE,
        null=True,
    )
    value = models.CharField(max_length=255, null=True)


class DeviceDefinitionProperty(TimeStampedModel):
    """DeviceDefinitionProperty model."""

    type = models.ForeignKey(
        CodeableConcept,
        related_name="device_definition_property_type",
        on_delete=models.CASCADE,
        null=True,
    )
    value_codeable_concept = models.ForeignKey(
        CodeableConcept,
        related_name="device_definition_property_value_code",
        on_delete=models.CASCADE,
        null=True,
    )
    value_string = models.CharField(max_length=255, null=True)
    value_boolean = models.BooleanField(null=True)
    value_integer = models.IntegerField(null=True)
    value_quantity = models.ForeignKey(
        Quantity,
        related_name="device_definition_property_value_quantity",
        on_delete=models.CASCADE,
        null=True,
    )
    value_range = models.ForeignKey(
        Range,
        related_name="device_definition_property_value_range",
        on_delete=models.CASCADE,
        null=True,
    )
    value_attachment = models.ForeignKey(
        Attachment,
        related_name="device_definition_property_value_attachment",
        on_delete=models.CASCADE,
        null=True,
    )


class DeviceDefinitionLink(TimeStampedModel):
    """DeviceDefinitionLink model."""

    relation = models.ForeignKey(
        Coding,
        related_name="device_definition_link_relation",
        on_delete=models.CASCADE,
        null=True,
    )
    related_device = models.ForeignKey(
        "DeviceDefinitionCodeableReference",
        related_name="device_definition_link_related_device",
        on_delete=models.CASCADE,
        null=True,
    )


class DeviceDefinitionMaterial(TimeStampedModel):
    """DeviceDefinitionMaterial model."""

    substance = models.ForeignKey(
        CodeableConcept,
        related_name="device_definition_material_substance",
        on_delete=models.CASCADE,
        null=True,
    )
    alternate = models.BooleanField(null=True)
    allergenic_indicator = models.BooleanField(null=True)


class DeviceDefinitionGuideline(TimeStampedModel):
    """DeviceDefinitionGuideline model."""

    use_context = models.ManyToManyField(
        UsageContext, related_name="device_definition_guideline_use_context", blank=True
    )
    usage_instruction = models.TextField(null=True)
    related_artifact = models.ManyToManyField(
        RelatedArtifact,
        related_name="device_definition_guideline_related_artifact",
        blank=True,
    )
    indication = models.ManyToManyField(
        CodeableConcept,
        related_name="device_definition_guideline_indication",
        blank=True,
    )
    contraindication = models.ManyToManyField(
        CodeableConcept,
        related_name="device_definition_guideline_contraindication",
        blank=True,
    )
    warning = models.ManyToManyField(
        CodeableConcept, related_name="device_definition_guideline_warning", blank=True
    )
    intended_use = models.CharField(max_length=255, null=True)


class DeviceDefinitionCorrectiveAction(TimeStampedModel):
    """DeviceDefinitionCorrectiveAction model."""

    recall = models.BooleanField(null=True)
    scope = models.CharField(
        max_length=255,
        null=True,
        choices=choices.DeviceDefinitionCorrectiveActionScope.choices,
    )
    period = models.ForeignKey(
        Period,
        related_name="device_definition_corrective_action_period",
        on_delete=models.CASCADE,
        null=True,
    )


class DeviceDefinitionChargeItem(TimeStampedModel):
    """DeviceDefinitionChargeItem model."""

    # charge_item_code = models.ForeignKey(
    #     "chargeitems.ChargeItemDefinitionCodeableReference",
    #     related_name="device_definition_charge_item_charge_item_code",
    #     on_delete=models.CASCADE,
    #     null=True,
    # )
    count = models.ForeignKey(
        Quantity,
        related_name="device_definition_charge_item_count",
        on_delete=models.CASCADE,
        null=True,
    )
    effective_period = models.ForeignKey(
        Period,
        related_name="device_definition_charge_item_effective_period",
        on_delete=models.CASCADE,
        null=True,
    )
    use_context = models.ManyToManyField(
        UsageContext,
        related_name="device_definition_charge_item_use_context",
        blank=True,
    )


class DeviceDefinition(TimeStampedModel):
    """DeviceDefinition model."""

    description = models.TextField(null=True)
    identifier = models.ManyToManyField(
        Identifier, related_name="device_definitions_identifier", blank=True
    )
    udi_device_identifier = models.ForeignKey(
        DeviceDefinitionUdiDeviceIdentifier,
        related_name="device_definitions_udi_device_identifier",
        on_delete=models.CASCADE,
        null=True,
    )
    regulatory_identifier = models.ManyToManyField(
        DeviceDefinitionRegulatoryIdentifier, blank=True
    )
    part_number = models.CharField(max_length=255, null=True)
    manufacturer = models.ForeignKey(
        OrganizationReference,
        related_name="device_definitions_manufacturer",
        on_delete=models.CASCADE,
        null=True,
    )
    device_name = models.ManyToManyField(
        DeviceDefinitionDeviceName,
        related_name="device_definitions_device_name",
        blank=True,
    )
    model_number = models.CharField(max_length=255, null=True)
    contact = models.ManyToManyField(
        ContactDetail, related_name="device_definitions_contact", blank=True
    )
    publisher = models.CharField(max_length=255, null=True)
    classification = models.ManyToManyField(
        DeviceDefinitionClassification,
        related_name="device_definitions_classification",
        blank=True,
    )
    conforms_to = models.ManyToManyField(
        DeviceDefinitionConformsTo,
        related_name="device_definitions_conforms_to",
        blank=True,
    )
    has_part = models.ManyToManyField(
        DeviceDefinitionHasPart, related_name="device_definitions_has_part", blank=True
    )
    packaging = models.ManyToManyField(
        DeviceDefinitionPackaging,
        related_name="device_definitions_packaging",
        blank=True,
    )
    # device_version = models.ManyToManyField(
    #     DeviceDefinitionDeviceVersion, related_name="device_definitions_device_version", blank=True
    # )
    safety = models.ManyToManyField(
        CodeableConcept, related_name="device_definitions_safety", blank=True
    )
    shelf_life_storage = models.ForeignKey(
        ProductShelfLife,
        related_name="device_definitions_shelf_life_storage",
        on_delete=models.CASCADE,
        null=True,
    )
    language_code = models.ForeignKey(
        CodeableConcept,
        related_name="device_definitions_language_code",
        on_delete=models.CASCADE,
        null=True,
    )
    property = models.ManyToManyField(
        DeviceDefinitionProperty, related_name="device_definitions_property", blank=True
    )
    link = models.ManyToManyField(
        DeviceDefinitionLink, related_name="device_definitions_link", blank=True
    )
    material = models.ManyToManyField(
        DeviceDefinitionMaterial, related_name="device_definitions_material", blank=True
    )
    guideline = models.ForeignKey(
        DeviceDefinitionGuideline,
        related_name="device_definitions_guideline",
        on_delete=models.CASCADE,
        null=True,
    )
    corrective_action = models.ForeignKey(
        DeviceDefinitionCorrectiveAction,
        related_name="device_definitions_corrective_action",
        on_delete=models.CASCADE,
        null=True,
    )
    charge_item = models.ManyToManyField(
        DeviceDefinitionChargeItem,
        related_name="device_definitions_charge_item",
        blank=True,
    )



"""supply delivery urls."""

from django.urls import path

from dfhir.supplydeliveries import views

app_name = "supplydelivery"

urlpatterns = [
    path("supplydelivery/", views.SupplyDeliveryListView.as_view()),
    path("supplydelivery/<int:pk>/", views.SupplyDeliveryDetailedView.as_view()),
]



"""Procedures urls."""

from django.urls import path

from . import views

app_name = "procedures"


urlpatterns = [
    path(
        "procedures/", views.ProcedureListView.as_view(), name="procedure-list-create"
    ),
    path(
        "procedures/<int:pk>/",
        views.ProcedureDetailView.as_view(),
        name="procedure-retrieve-update-destroy",
    ),
]
